@@ -47,8 +47,9 @@ Example Provider Configuration
 # pylint: disable=invalid-name,function-redefined
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import os
+import sys
 import re
 import pprint
 import logging
@@ -58,6 +59,7 @@ from salt.utils.versions import LooseVersion as _LooseVersion
 
 # Import 3rd-party libs
 # pylint: disable=import-error
+LIBCLOUD_IMPORT_ERROR = None
 try:
     import libcloud
     from libcloud.compute.types import Provider
@@ -68,25 +70,28 @@ try:
         ResourceInUseError,
         ResourceNotFoundError,
         )
-    # This work-around for Issue #32743 is no longer needed for libcloud >= 1.4.0.
-    # However, older versions of libcloud must still be supported with this work-around.
-    # This work-around can be removed when the required minimum version of libcloud is
-    # 2.0.0 (See PR #40837 - which is implemented in Salt Oxygen).
+    # This work-around for Issue #32743 is no longer needed for libcloud >=
+    # 1.4.0. However, older versions of libcloud must still be supported with
+    # this work-around. This work-around can be removed when the required
+    # minimum version of libcloud is 2.0.0 (See PR #40837 - which is
+    # implemented in Salt 2018.3.0).
     if _LooseVersion(libcloud.__version__) < _LooseVersion('1.4.0'):
         # See https://github.com/saltstack/salt/issues/32743
         import libcloud.security
         libcloud.security.CA_CERTS_PATH.append('/etc/ssl/certs/YaST-CA.pem')
     HAS_LIBCLOUD = True
 except ImportError:
+    LIBCLOUD_IMPORT_ERROR = sys.exc_info()
     HAS_LIBCLOUD = False
 # pylint: enable=import-error
 
 # Import salt libs
-from salt.utils import namespaced_function
-import salt.ext.six as six
+from salt.utils.functools import namespaced_function
+from salt.ext import six
 import salt.utils.cloud
+import salt.utils.files
+import salt.utils.http
 import salt.config as config
-from salt.utils import http
 from salt.cloud.libcloudfuncs import *  # pylint: disable=redefined-builtin,wildcard-import,unused-wildcard-import
 from salt.exceptions import (
     SaltCloudSystemExit,
@@ -129,7 +134,8 @@ def __virtual__():
 
         parameters = details['gce']
         pathname = os.path.expanduser(parameters['service_account_private_key'])
-        if salt.utils.cloud.check_key_path_and_mode(
+        # empty pathname will tell libcloud to use instance credentials
+        if pathname and salt.utils.cloud.check_key_path_and_mode(
                 provider, pathname
         ) is False:
             return False
@@ -154,6 +160,9 @@ def get_dependencies():
     '''
     Warn if dependencies aren't met.
     '''
+    if LIBCLOUD_IMPORT_ERROR:
+        log.error("Failure when importing LibCloud: ", exc_info=LIBCLOUD_IMPORT_ERROR)
+        log.error("Note: The libcloud dependency is called 'apache-libcloud' on PyPi/pip.")
     return config.check_driver_dependencies(
         __virtualname__,
         {'libcloud': HAS_LIBCLOUD}
@@ -440,10 +449,10 @@ def __get_host(node, vm_):
     '''
     if __get_ssh_interface(vm_) == 'private_ips' or vm_['external_ip'] is None:
         ip_address = node.private_ips[0]
-        log.info('Salt node data. Private_ip: {0}'.format(ip_address))
+        log.info('Salt node data. Private_ip: %s', ip_address)
     else:
         ip_address = node.public_ips[0]
-        log.info('Salt node data. Public_ip: {0}'.format(ip_address))
+        log.info('Salt node data. Public_ip: %s', ip_address)
 
     if len(ip_address) > 0:
         return ip_address
@@ -544,7 +553,7 @@ def _parse_allow(allow):
         if len(seen_protos[k]) > 0:
             d['ports'] = seen_protos[k]
         allow_dict.append(d)
-    log.debug("firewall allowed protocols/ports: {0}".format(allow_dict))
+    log.debug("firewall allowed protocols/ports: %s", allow_dict)
     return allow_dict
 
 
@@ -668,9 +677,8 @@ def delete_network(kwargs=None, call=None):
         )
     except ResourceNotFoundError as exc:
         log.error(
-            'Nework {0} was not found. Exception was: {1}'.format(
-                name, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Nework %s was not found. Exception was: %s',
+            name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -841,9 +849,8 @@ def delete_subnetwork(kwargs=None, call=None):
         result = conn.ex_destroy_subnetwork(name, region)
     except ResourceNotFoundError as exc:
         log.error(
-            'Subnetwork {0} was not found. Exception was: {1}'.format(
-                name, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Subnetwork %s was not found. Exception was: %s',
+            name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -1014,9 +1021,8 @@ def delete_fwrule(kwargs=None, call=None):
         )
     except ResourceNotFoundError as exc:
         log.error(
-            'Rule {0} was not found. Exception was: {1}'.format(
-                name, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Rule %s was not found. Exception was: %s',
+            name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -1174,9 +1180,8 @@ def delete_hc(kwargs=None, call=None):
         )
     except ResourceNotFoundError as exc:
         log.error(
-            'Health check {0} was not found. Exception was: {1}'.format(
-                name, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Health check %s was not found. Exception was: %s',
+            name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -1323,9 +1328,8 @@ def delete_address(kwargs=None, call=None):
         )
     except ResourceNotFoundError as exc:
         log.error(
-            'Address {0} in region {1} was not found. Exception was: {2}'.format(
-                name, ex_region, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Address %s in region %s was not found. Exception was: %s',
+            name, ex_region, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -1499,9 +1503,8 @@ def delete_lb(kwargs=None, call=None):
         )
     except ResourceNotFoundError as exc:
         log.error(
-            'Load balancer {0} was not found. Exception was: {1}'.format(
-                name, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Load balancer %s was not found. Exception was: %s',
+            name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -1635,9 +1638,8 @@ def detach_lb(kwargs=None, call=None):
 
     if not remove_member:
         log.error(
-            'The specified member {0} was not a member of LB {1}.'.format(
-                kwargs['member'], kwargs['name']
-            )
+            'The specified member %s was not a member of LB %s.',
+            kwargs['member'], kwargs['name']
         )
         return False
 
@@ -1704,9 +1706,8 @@ def delete_snapshot(kwargs=None, call=None):
         )
     except ResourceNotFoundError as exc:
         log.error(
-            'Snapshot {0} was not found. Exception was: {1}'.format(
-                name, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Snapshot %s was not found. Exception was: %s',
+            name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -1765,10 +1766,9 @@ def delete_disk(kwargs=None, call=None):
         result = conn.destroy_volume(disk)
     except ResourceInUseError as exc:
         log.error(
-            'Disk {0} is in use and must be detached before deleting.\n'
-            'The following exception was thrown by libcloud:\n{1}'.format(
-                disk.name, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Disk %s is in use and must be detached before deleting.\n'
+            'The following exception was thrown by libcloud:\n%s',
+            disk.name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -1909,9 +1909,8 @@ def create_snapshot(kwargs=None, call=None):
         disk = conn.ex_get_volume(disk_name)
     except ResourceNotFoundError as exc:
         log.error(
-            'Disk {0} was not found. Exception was: {1}'.format(
-                disk_name, exc),
-            exc_info_on_loglevel=logging.DEBUG
+            'Disk %s was not found. Exception was: %s',
+            disk_name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -2079,6 +2078,7 @@ def attach_disk(name=None, kwargs=None, call=None):
     disk_name = kwargs['disk_name']
     mode = kwargs.get('mode', 'READ_WRITE').upper()
     boot = kwargs.get('boot', False)
+    auto_delete = kwargs.get('auto_delete', False)
     if boot and boot.lower() in ['true', 'yes', 'enabled']:
         boot = True
     else:
@@ -2108,7 +2108,8 @@ def attach_disk(name=None, kwargs=None, call=None):
         transport=__opts__['transport']
     )
 
-    result = conn.attach_volume(node, disk, ex_mode=mode, ex_boot=boot)
+    result = conn.attach_volume(node, disk, ex_mode=mode, ex_boot=boot,
+                                ex_auto_delete=auto_delete)
 
     __utils__['cloud.fire_event'](
         'event',
@@ -2278,12 +2279,10 @@ def destroy(vm_name, call=None):
         node = conn.ex_get_node(vm_name)
     except Exception as exc:  # pylint: disable=W0703
         log.error(
-            'Could not locate instance {0}\n\n'
+            'Could not locate instance %s\n\n'
             'The following exception was thrown by libcloud when trying to '
-            'run the initial deployment: \n{1}'.format(
-                vm_name, exc
-            ),
-            exc_info_on_loglevel=logging.DEBUG
+            'run the initial deployment: \n%s',
+            vm_name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         raise SaltCloudSystemExit(
             'Could not find instance {0}.'.format(vm_name)
@@ -2317,12 +2316,10 @@ def destroy(vm_name, call=None):
         inst_deleted = conn.destroy_node(node)
     except Exception as exc:  # pylint: disable=W0703
         log.error(
-            'Could not destroy instance {0}\n\n'
+            'Could not destroy instance %s\n\n'
             'The following exception was thrown by libcloud when trying to '
-            'run the initial deployment: \n{1}'.format(
-                vm_name, exc
-            ),
-            exc_info_on_loglevel=logging.DEBUG
+            'run the initial deployment: \n%s',
+            vm_name, exc, exc_info_on_loglevel=logging.DEBUG
         )
         raise SaltCloudSystemExit(
             'Could not destroy instance {0}.'.format(vm_name)
@@ -2356,12 +2353,10 @@ def destroy(vm_name, call=None):
             # to allow completion of instance deletion.  Just log the error
             # and keep going.
             log.error(
-                'Could not destroy disk {0}\n\n'
+                'Could not destroy disk %s\n\n'
                 'The following exception was thrown by libcloud when trying '
-                'to run the initial deployment: \n{1}'.format(
-                    vm_name, exc
-                ),
-                exc_info_on_loglevel=logging.DEBUG
+                'to run the initial deployment: \n%s',
+                vm_name, exc, exc_info_on_loglevel=logging.DEBUG
             )
         __utils__['cloud.fire_event'](
             'event',
@@ -2380,19 +2375,30 @@ def destroy(vm_name, call=None):
 
 def create_attach_volumes(name, kwargs, call=None):
     '''
+    .. versionadded:: 2017.7.0
+
     Create and attach multiple volumes to a node. The 'volumes' and 'node'
     arguments are required, where 'node' is a libcloud node, and 'volumes'
     is a list of maps, where each map contains:
 
-    'size': The size of the new disk in GB. Required.
-    'type': The disk type, either pd-standard or pd-ssd. Optional, defaults to pd-standard.
-    'image': An image to use for this new disk. Optional.
-    'snapshot': A snapshot to use for this new disk. Optional.
+    size
+        The size of the new disk in GB. Required.
+
+    type
+        The disk type, either pd-standard or pd-ssd. Optional, defaults to pd-standard.
+
+    image
+        An image to use for this new disk. Optional.
+
+    snapshot
+        A snapshot to use for this new disk. Optional.
+
+    auto_delete
+        An option(bool) to keep or remove the disk upon instance deletion.
+        Optional, defaults to False.
 
     Volumes are attached in the order in which they are given, thus on a new
     node the first volume will be /dev/sdb, the second /dev/sdc, and so on.
-
-    .. versionadded:: 2017.7.0
     '''
     if call != 'action':
         raise SaltCloudSystemExit(
@@ -2400,9 +2406,10 @@ def create_attach_volumes(name, kwargs, call=None):
             '-a or --action.'
         )
 
-    volumes = kwargs['volumes']
+    volumes = literal_eval(kwargs['volumes'])
     node = kwargs['node']
-    node_data = _expand_node(node)
+    conn = get_conn()
+    node_data = _expand_node(conn.ex_get_node(node))
     letter = ord('a') - 1
 
     for idx, volume in enumerate(volumes):
@@ -2412,9 +2419,10 @@ def create_attach_volumes(name, kwargs, call=None):
           'disk_name': volume_name,
           'location': node_data['extra']['zone']['name'],
           'size': volume['size'],
-          'type': volume['type'],
-          'image': volume['image'],
-          'snapshot': volume['snapshot']
+          'type': volume.get('type', 'pd-standard'),
+          'image': volume.get('image', None),
+          'snapshot': volume.get('snapshot', None),
+          'auto_delete': volume.get('auto_delete', False)
         }
 
         create_disk(volume_dict, 'function')
@@ -2500,10 +2508,11 @@ def request_instance(vm_):
                 '\'pd-standard\', \'pd-ssd\''
             )
 
-    log.info('Creating GCE instance {0} in {1}'.format(vm_['name'],
-        kwargs['location'].name)
+    log.info(
+        'Creating GCE instance %s in %s',
+        vm_['name'], kwargs['location'].name
     )
-    log.debug('Create instance kwargs {0}'.format(str(kwargs)))
+    log.debug('Create instance kwargs %s', kwargs)
 
     __utils__['cloud.fire_event'](
         'event',
@@ -2518,12 +2527,10 @@ def request_instance(vm_):
         node_data = conn.create_node(**kwargs)
     except Exception as exc:  # pylint: disable=W0703
         log.error(
-            'Error creating {0} on GCE\n\n'
+            'Error creating %s on GCE\n\n'
             'The following exception was thrown by libcloud when trying to '
-            'run the initial deployment: \n{1}'.format(
-                vm_['name'], exc
-            ),
-            exc_info_on_loglevel=logging.DEBUG
+            'run the initial deployment: \n%s',
+            vm_['name'], exc, exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -2541,7 +2548,7 @@ def request_instance(vm_):
             transport=__opts__['transport']
         )
 
-        log.info('Create and attach volumes to node {0}'.format(vm_['name']))
+        log.info('Create and attach volumes to node %s', vm_['name'])
         create_attach_volumes(
             vm_['name'],
             {
@@ -2580,13 +2587,15 @@ def create(vm_=None, call=None):
     ssh_user, ssh_key = __get_ssh_credentials(vm_)
     vm_['ssh_host'] = __get_host(node_data, vm_)
     vm_['key_filename'] = ssh_key
-    __utils__['cloud.bootstrap'](vm_, __opts__)
 
-    log.info('Created Cloud VM \'{0[name]}\''.format(vm_))
+    ret = __utils__['cloud.bootstrap'](vm_, __opts__)
+
+    ret.update(node_dict)
+
+    log.info('Created Cloud VM \'%s\'', vm_['name'])
     log.trace(
-        '\'{0[name]}\' VM creation details:\n{1}'.format(
-            vm_, pprint.pformat(node_dict)
-        )
+        '\'%s\' VM creation details:\n%s',
+        vm_['name'], pprint.pformat(node_dict)
     )
 
     __utils__['cloud.fire_event'](
@@ -2598,7 +2607,7 @@ def create(vm_=None, call=None):
         transport=__opts__['transport']
     )
 
-    return node_dict
+    return ret
 
 
 def update_pricing(kwargs=None, call=None):
@@ -2614,12 +2623,12 @@ def update_pricing(kwargs=None, call=None):
     .. versionadded:: 2015.8.0
     '''
     url = 'https://cloudpricingcalculator.appspot.com/static/data/pricelist.json'
-    price_json = http.query(url, decode=True, decode_type='json')
+    price_json = salt.utils.http.query(url, decode=True, decode_type='json')
 
     outfile = os.path.join(
         __opts__['cachedir'], 'gce-pricing.p'
     )
-    with salt.utils.fopen(outfile, 'w') as fho:
+    with salt.utils.files.fopen(outfile, 'w') as fho:
         msgpack.dump(price_json['dict'], fho)
 
     return True
@@ -2642,7 +2651,7 @@ def show_pricing(kwargs=None, call=None):
     if not profile:
         return {'Error': 'The requested profile was not found'}
 
-    # Make sure the profile belongs to Digital Ocean
+    # Make sure the profile belongs to DigitalOcean
     provider = profile.get('provider', '0:0')
     comps = provider.split(':')
     if len(comps) < 2 or comps[1] != 'gce':
@@ -2658,7 +2667,7 @@ def show_pricing(kwargs=None, call=None):
     if not os.path.exists(pricefile):
         update_pricing()
 
-    with salt.utils.fopen(pricefile, 'r') as fho:
+    with salt.utils.files.fopen(pricefile, 'r') as fho:
         sizes = msgpack.load(fho)
 
     per_hour = float(sizes['gcp_price_list'][size][region])
