@@ -12,49 +12,127 @@ Support for Opkg
 
 .. note::
 
-    For version comparison support, the ``opkg-utils`` package must be
-    installed.
+    For version comparison support on opkg < 0.3.4, the ``opkg-utils`` package
+    must be installed.
 
 '''
-from __future__ import absolute_import
-
 # Import python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import copy
 import os
 import re
 import logging
-from salt.ext import six
-try:
-    from shlex import quote as _cmd_quote  # pylint: disable=E0611
-except ImportError:
-    from pipes import quote as _cmd_quote
+import errno
 
 # Import salt libs
-import salt.utils
-import salt.utils.pkg
+import salt.utils.args
+import salt.utils.data
+import salt.utils.files
 import salt.utils.itertools
-from salt.utils.decorators import which as _which
-
+import salt.utils.path
+import salt.utils.pkg
+import salt.utils.stringutils
+import salt.utils.versions
 from salt.exceptions import (
     CommandExecutionError, MinionError, SaltInvocationError
 )
+# Import 3rd-party libs
+from salt.ext import six
+from salt.ext.six.moves import shlex_quote as _cmd_quote  # pylint: disable=import-error
 
-REPO_REGEXP = r'^#?\s*(src|src/gz)\s+[^\s<>]+\s+[^\s<>]+'
+REPO_REGEXP = r'^#?\s*(src|src/gz)\s+([^\s<>]+|"[^<>]+")\s+[^\s<>]+'
 OPKG_CONFDIR = '/etc/opkg'
+ATTR_MAP = {
+    'Architecture': 'arch',
+    'Homepage': 'url',
+    'Installed-Time': 'install_date_time_t',
+    'Maintainer': 'packager',
+    'Package': 'name',
+    'Section': 'group'
+}
 
 log = logging.getLogger(__name__)
 
 # Define the module's virtual name
 __virtualname__ = 'pkg'
 
+NILRT_RESTARTCHECK_STATE_PATH = '/var/lib/salt/restartcheck_state'
+
+
+def _update_nilrt_restart_state():
+    '''
+    NILRT systems determine whether to reboot after various package operations
+    including but not limited to kernel module installs/removals by checking
+    specific file md5sums & timestamps. These files are touched/modified by
+    the post-install/post-remove functions of their respective packages.
+
+    The opkg module uses this function to store/update those file timestamps
+    and checksums to be used later by the restartcheck module.
+
+    '''
+    __salt__['cmd.shell']('stat -c %Y /lib/modules/$(uname -r)/modules.dep >{0}/modules.dep.timestamp'
+                          .format(NILRT_RESTARTCHECK_STATE_PATH))
+    __salt__['cmd.shell']('md5sum /lib/modules/$(uname -r)/modules.dep >{0}/modules.dep.md5sum'
+                          .format(NILRT_RESTARTCHECK_STATE_PATH))
+
+    # We can't assume nisysapi.ini always exists like modules.dep
+    nisysapi_path = '/usr/local/natinst/share/nisysapi.ini'
+    if os.path.exists(nisysapi_path):
+        __salt__['cmd.shell']('stat -c %Y {0} >{1}/nisysapi.ini.timestamp'
+                              .format(nisysapi_path, NILRT_RESTARTCHECK_STATE_PATH))
+        __salt__['cmd.shell']('md5sum {0} >{1}/nisysapi.ini.md5sum'
+                              .format(nisysapi_path, NILRT_RESTARTCHECK_STATE_PATH))
+
+
+def _get_restartcheck_result(errors):
+    '''
+    Return restartcheck result and append errors (if any) to ``errors``
+    '''
+    rs_result = __salt__['restartcheck.restartcheck'](verbose=False)
+    if isinstance(rs_result, dict) and 'comment' in rs_result:
+        errors.append(rs_result['comment'])
+    return rs_result
+
+
+def _process_restartcheck_result(rs_result):
+    '''
+    Check restartcheck output to see if system/service restarts were requested
+    and take appropriate action.
+    '''
+    if 'No packages seem to need to be restarted' in rs_result:
+        return
+    for rstr in rs_result:
+        if 'System restart required' in rstr:
+            _update_nilrt_restart_state()
+            __salt__['system.set_reboot_required_witnessed']()
+        else:
+            service = os.path.join('/etc/init.d', rstr)
+            if os.path.exists(service):
+                __salt__['cmd.run']([service, 'restart'])
+
 
 def __virtual__():
     '''
     Confirm this module is on a nilrt based system
     '''
-    if __grains__.get('os_family', False) == 'NILinuxRT':
+    if __grains__.get('os_family') == 'NILinuxRT':
+        try:
+            os.makedirs(NILRT_RESTARTCHECK_STATE_PATH)
+        except OSError as exc:
+            if exc.errno != errno.EEXIST:
+                return False, 'Error creating {0} (-{1}): {2}'.format(
+                    NILRT_RESTARTCHECK_STATE_PATH,
+                    exc.errno,
+                    exc.strerror)
+        # modules.dep always exists, make sure it's restart state files also exist
+        if not (os.path.exists(os.path.join(NILRT_RESTARTCHECK_STATE_PATH, 'modules.dep.timestamp')) and
+                os.path.exists(os.path.join(NILRT_RESTARTCHECK_STATE_PATH, 'modules.dep.md5sum'))):
+            _update_nilrt_restart_state()
         return __virtualname__
-    return (False, "Module opkg only works on nilrt based systems")
+
+    if os.path.isdir(OPKG_CONFDIR):
+        return __virtualname__
+    return False, "Module opkg only works on OpenEmbedded based systems"
 
 
 def latest_version(*names, **kwargs):
@@ -74,7 +152,7 @@ def latest_version(*names, **kwargs):
         salt '*' pkg.latest_version <package name>
         salt '*' pkg.latest_version <package1> <package2> <package3> ...
     '''
-    refresh = salt.utils.is_true(kwargs.pop('refresh', True))
+    refresh = salt.utils.data.is_true(kwargs.pop('refresh', True))
 
     if len(names) == 0:
         return ''
@@ -125,7 +203,7 @@ def version(*names, **kwargs):
     return __salt__['pkg_resource.version'](*names, **kwargs)
 
 
-def refresh_db():
+def refresh_db(failhard=False, **kwargs):  # pylint: disable=unused-argument
     '''
     Updates the opkg database to latest packages based upon repositories
 
@@ -135,6 +213,14 @@ def refresh_db():
     - ``True``: Database updated successfully
     - ``False``: Problem updating database
 
+    failhard
+        If False, return results of failed lines as ``False`` for the package
+        database that encountered the error.
+        If True, raise an error with a list of the package databases that
+        encountered errors.
+
+        .. versionadded:: 2018.3.0
+
     CLI Example:
 
     .. code-block:: bash
@@ -144,37 +230,50 @@ def refresh_db():
     # Remove rtag file to keep multiple refreshes from happening in pkg states
     salt.utils.pkg.clear_rtag(__opts__)
     ret = {}
+    error_repos = []
     cmd = ['opkg', 'update']
+    # opkg returns a non-zero retcode when there is a failure to refresh
+    # from one or more repos. Due to this, ignore the retcode.
     call = __salt__['cmd.run_all'](cmd,
                                    output_loglevel='trace',
-                                   python_shell=False)
-    if call['retcode'] != 0:
-        comment = ''
-        if 'stderr' in call:
-            comment += call['stderr']
-
-        raise CommandExecutionError(
-            '{0}'.format(comment)
-        )
-    else:
-        out = call['stdout']
+                                   python_shell=False,
+                                   ignore_retcode=True,
+                                   redirect_stderr=True)
 
+    out = call['stdout']
+    prev_line = ''
     for line in salt.utils.itertools.split(out, '\n'):
         if 'Inflating' in line:
-            key = line.strip().split()[1].split('.')[0]
+            key = line.strip().split()[1][:-1]
+            ret[key] = True
+        elif 'Updated source' in line:
+            # Use the previous line.
+            key = prev_line.strip().split()[1][:-1]
             ret[key] = True
         elif 'Failed to download' in line:
             key = line.strip().split()[5].split(',')[0]
             ret[key] = False
+            error_repos.append(key)
+        prev_line = line
+
+    if failhard and error_repos:
+        raise CommandExecutionError(
+            'Error getting repos: {0}'.format(', '.join(error_repos))
+        )
+
+    # On a non-zero exit code where no failed repos were found, raise an
+    # exception because this appears to be a different kind of error.
+    if call['retcode'] != 0 and not error_repos:
+        raise CommandExecutionError(out)
+
     return ret
 
 
-# TODO: opkg doesn't support installation of a specific version of a package
-#       (opkg issue 176). Once fixed, this function should add support.
 def install(name=None,
             refresh=False,
             pkgs=None,
             sources=None,
+            reinstall=False,
             **kwargs):
     '''
     Install the passed package, add refresh=True to update the opkg database.
@@ -195,6 +294,23 @@ def install(name=None,
     refresh
         Whether or not to refresh the package database before installing.
 
+    version
+        Install a specific version of the package, e.g. 1.2.3~0ubuntu0. Ignored
+        if "pkgs" or "sources" is passed.
+
+        .. versionadded:: 2017.7.0
+
+    reinstall : False
+        Specifying reinstall=True will use ``opkg install --force-reinstall``
+        rather than simply ``opkg install`` for requested packages that are
+        already installed.
+
+        If a version is specified with the requested package, then ``opkg
+        install --force-reinstall`` will only be used if the installed version
+        matches the requested version.
+
+        .. versionadded:: 2017.7.0
+
 
     Multiple Package Installation Options:
 
@@ -207,6 +323,7 @@ def install(name=None,
         .. code-block:: bash
 
             salt '*' pkg.install pkgs='["foo", "bar"]'
+            salt '*' pkg.install pkgs='["foo", {"bar": "1.2.3-0ubuntu0"}]'
 
     sources
         A list of IPK packages to install. Must be passed as a list of dicts,
@@ -223,49 +340,149 @@ def install(name=None,
     install_recommends
         Whether to install the packages marked as recommended. Default is True.
 
+    only_upgrade
+        Only upgrade the packages (disallow downgrades), if they are already
+        installed. Default is False.
+
+        .. versionadded:: 2017.7.0
+
     Returns a dict containing the new package names and versions::
 
         {'<package>': {'old': '<old-version>',
                        'new': '<new-version>'}}
     '''
-    refreshdb = salt.utils.is_true(refresh)
+    refreshdb = salt.utils.data.is_true(refresh)
 
     try:
-        pkgs, pkg_type = __salt__['pkg_resource.parse_targets'](
+        pkg_params, pkg_type = __salt__['pkg_resource.parse_targets'](
             name, pkgs, sources, **kwargs
         )
     except MinionError as exc:
         raise CommandExecutionError(exc)
 
     old = list_pkgs()
-    cmd = ['opkg', 'install']
-    if pkgs is None or len(pkgs) == 0:
+    cmd_prefix = ['opkg', 'install']
+    to_install = []
+    to_reinstall = []
+    to_downgrade = []
+
+    if pkg_params is None or len(pkg_params) == 0:
         return {}
     elif pkg_type == 'file':
-        cmd.extend(pkgs)
+        if reinstall:
+            cmd_prefix.append('--force-reinstall')
+        if not kwargs.get('only_upgrade', False):
+            cmd_prefix.append('--force-downgrade')
+        to_install.extend(pkg_params)
     elif pkg_type == 'repository':
-        targets = list(pkgs.keys())
-        if 'install_recommends' in kwargs and not kwargs['install_recommends']:
-            cmd.append('--no-install-recommends')
-        cmd.extend(targets)
+        if not kwargs.get('install_recommends', True):
+            cmd_prefix.append('--no-install-recommends')
+        for pkgname, pkgversion in six.iteritems(pkg_params):
+            if (name and pkgs is None and kwargs.get('version') and
+                    len(pkg_params) == 1):
+                # Only use the 'version' param if 'name' was not specified as a
+                # comma-separated list
+                version_num = kwargs['version']
+            else:
+                version_num = pkgversion
+
+            if version_num is None:
+                # Don't allow downgrades if the version
+                # number is not specified.
+                if reinstall and pkgname in old:
+                    to_reinstall.append(pkgname)
+                else:
+                    to_install.append(pkgname)
+            else:
+                pkgstr = '{0}={1}'.format(pkgname, version_num)
+                cver = old.get(pkgname, '')
+                if reinstall and cver and salt.utils.versions.compare(
+                        ver1=version_num,
+                        oper='==',
+                        ver2=cver,
+                        cmp_func=version_cmp):
+                    to_reinstall.append(pkgstr)
+                elif not cver or salt.utils.versions.compare(
+                        ver1=version_num,
+                        oper='>=',
+                        ver2=cver,
+                        cmp_func=version_cmp):
+                    to_install.append(pkgstr)
+                else:
+                    if not kwargs.get('only_upgrade', False):
+                        to_downgrade.append(pkgstr)
+                    else:
+                        # This should cause the command to fail.
+                        to_install.append(pkgstr)
+
+    cmds = []
+
+    if to_install:
+        cmd = copy.deepcopy(cmd_prefix)
+        cmd.extend(to_install)
+        cmds.append(cmd)
+
+    if to_downgrade:
+        cmd = copy.deepcopy(cmd_prefix)
+        cmd.append('--force-downgrade')
+        cmd.extend(to_downgrade)
+        cmds.append(cmd)
+
+    if to_reinstall:
+        cmd = copy.deepcopy(cmd_prefix)
+        cmd.append('--force-reinstall')
+        cmd.extend(to_reinstall)
+        cmds.append(cmd)
+
+    if not cmds:
+        return {}
 
     if refreshdb:
         refresh_db()
 
-    out = __salt__['cmd.run_all'](
-        cmd,
-        output_loglevel='trace',
-        python_shell=False
-    )
-
-    if out['retcode'] != 0 and out['stderr']:
-        errors = [out['stderr']]
-    else:
-        errors = []
+    errors = []
+    for cmd in cmds:
+        out = __salt__['cmd.run_all'](
+            cmd,
+            output_loglevel='trace',
+            python_shell=False
+        )
+        if out['retcode'] != 0:
+            if out['stderr']:
+                errors.append(out['stderr'])
+            else:
+                errors.append(out['stdout'])
 
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
+
+    if pkg_type == 'file' and reinstall:
+        # For file-based packages, prepare 'to_reinstall' to have a list
+        # of all the package names that may have been reinstalled.
+        # This way, we could include reinstalled packages in 'ret'.
+        for pkgfile in to_install:
+            # Convert from file name to package name.
+            cmd = ['opkg', 'info', pkgfile]
+            out = __salt__['cmd.run_all'](
+                cmd,
+                output_loglevel='trace',
+                python_shell=False
+            )
+            if out['retcode'] == 0:
+                # Just need the package name.
+                pkginfo_dict = _process_info_installed_output(
+                    out['stdout'], []
+                )
+                if pkginfo_dict:
+                    to_reinstall.append(list(pkginfo_dict.keys())[0])
+
+    for pkgname in to_reinstall:
+        if pkgname not in ret or pkgname in old:
+            ret.update({pkgname: {'old': old.get(pkgname, ''),
+                                  'new': new.get(pkgname, '')}})
+
+    rs_result = _get_restartcheck_result(errors)
 
     if errors:
         raise CommandExecutionError(
@@ -273,6 +490,8 @@ def install(name=None,
             info={'errors': errors, 'changes': ret}
         )
 
+    _process_restartcheck_result(rs_result)
+
     return ret
 
 
@@ -290,6 +509,15 @@ def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=unused-argument
         A list of packages to delete. Must be passed as a python list. The
         ``name`` parameter will be ignored if this option is passed.
 
+    remove_dependencies
+        Remove package and all dependencies
+
+        .. versionadded:: 2019.2.0
+
+    auto_remove_deps
+        Remove packages that were installed automatically to satisfy dependencies
+
+        .. versionadded:: 2019.2.0
 
     Returns a dict containing the changes.
 
@@ -300,6 +528,7 @@ def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=unused-argument
         salt '*' pkg.remove <package name>
         salt '*' pkg.remove <package1>,<package2>,<package3>
         salt '*' pkg.remove pkgs='["foo", "bar"]'
+        salt '*' pkg.remove pkgs='["foo", "bar"]' remove_dependencies=True auto_remove_deps=True
     '''
     try:
         pkg_params = __salt__['pkg_resource.parse_targets'](name, pkgs)[0]
@@ -311,6 +540,10 @@ def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=unused-argument
     if not targets:
         return {}
     cmd = ['opkg', 'remove']
+    if kwargs.get('remove_dependencies', False):
+        cmd.append('--force-removal-of-dependent-packages')
+    if kwargs.get('auto_remove_deps', False):
+        cmd.append('--autoremove')
     cmd.extend(targets)
 
     out = __salt__['cmd.run_all'](
@@ -318,14 +551,19 @@ def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=unused-argument
         output_loglevel='trace',
         python_shell=False
     )
-    if out['retcode'] != 0 and out['stderr']:
-        errors = [out['stderr']]
+    if out['retcode'] != 0:
+        if out['stderr']:
+            errors = [out['stderr']]
+        else:
+            errors = [out['stdout']]
     else:
         errors = []
 
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
+
+    rs_result = _get_restartcheck_result(errors)
 
     if errors:
         raise CommandExecutionError(
@@ -333,6 +571,8 @@ def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=unused-argument
             info={'errors': errors, 'changes': ret}
         )
 
+    _process_restartcheck_result(rs_result)
+
     return ret
 
 
@@ -365,7 +605,7 @@ def purge(name=None, pkgs=None, **kwargs):  # pylint: disable=unused-argument
     return remove(name=name, pkgs=pkgs)
 
 
-def upgrade(refresh=True):
+def upgrade(refresh=True, **kwargs):  # pylint: disable=unused-argument
     '''
     Upgrades all packages via ``opkg upgrade``
 
@@ -388,7 +628,9 @@ def upgrade(refresh=True):
            'comment': '',
            }
 
-    if salt.utils.is_true(refresh):
+    errors = []
+
+    if salt.utils.data.is_true(refresh):
         refresh_db()
 
     old = list_pkgs()
@@ -399,14 +641,21 @@ def upgrade(refresh=True):
                                      python_shell=False)
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
 
     if result['retcode'] != 0:
+        errors.append(result)
+
+    rs_result = _get_restartcheck_result(errors)
+
+    if errors:
         raise CommandExecutionError(
             'Problem encountered upgrading packages',
-            info={'changes': ret, 'result': result}
+            info={'errors': errors, 'changes': ret}
         )
 
+    _process_restartcheck_result(rs_result)
+
     return ret
 
 
@@ -592,9 +841,7 @@ def _set_state(pkg, state):
     ret = {}
     valid_states = ('hold', 'noprune', 'user', 'ok', 'installed', 'unpacked')
     if state not in valid_states:
-        raise SaltInvocationError(
-            'Invalid state: {0}'.format(state)
-        )
+        raise SaltInvocationError('Invalid state: {0}'.format(state))
     oldstate = _get_state(pkg)
     cmd = ['opkg', 'flag']
     cmd.append(state)
@@ -620,9 +867,9 @@ def list_pkgs(versions_as_list=False, **kwargs):
         salt '*' pkg.list_pkgs
         salt '*' pkg.list_pkgs versions_as_list=True
     '''
-    versions_as_list = salt.utils.is_true(versions_as_list)
+    versions_as_list = salt.utils.data.is_true(versions_as_list)
     # not yet implemented or not applicable
-    if any([salt.utils.is_true(kwargs.get(x))
+    if any([salt.utils.data.is_true(kwargs.get(x))
             for x in ('removed', 'purge_desired')]):
         return {}
 
@@ -638,7 +885,13 @@ def list_pkgs(versions_as_list=False, **kwargs):
     ret = {}
     out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
     for line in salt.utils.itertools.split(out, '\n'):
-        pkg_name, pkg_version = line.split(' - ')
+        # This is a continuation of package description
+        if not line or line[0] == ' ':
+            continue
+
+        # This contains package name, version, and description.
+        # Extract the first two.
+        pkg_name, pkg_version = line.split(' - ', 2)[:2]
         __salt__['pkg_resource.add_pkg'](ret, pkg_name, pkg_version)
 
     __salt__['pkg_resource.sort_pkglist'](ret)
@@ -648,7 +901,7 @@ def list_pkgs(versions_as_list=False, **kwargs):
     return ret
 
 
-def list_upgrades(refresh=True):
+def list_upgrades(refresh=True, **kwargs):  # pylint: disable=unused-argument
     '''
     List all available package upgrades.
 
@@ -659,7 +912,7 @@ def list_upgrades(refresh=True):
         salt '*' pkg.list_upgrades
     '''
     ret = {}
-    if salt.utils.is_true(refresh):
+    if salt.utils.data.is_true(refresh):
         refresh_db()
 
     cmd = ['opkg', 'list-upgradable']
@@ -673,9 +926,7 @@ def list_upgrades(refresh=True):
             comment += call['stderr']
         if 'stdout' in call:
             comment += call['stdout']
-        raise CommandExecutionError(
-                '{0}'.format(comment)
-        )
+        raise CommandExecutionError(comment)
     else:
         out = call['stdout']
 
@@ -686,7 +937,138 @@ def list_upgrades(refresh=True):
     return ret
 
 
-def upgrade_available(name):
+def _convert_to_standard_attr(attr):
+    '''
+    Helper function for _process_info_installed_output()
+
+    Converts an opkg attribute name to a standard attribute
+    name which is used across 'pkg' modules.
+    '''
+    ret_attr = ATTR_MAP.get(attr, None)
+    if ret_attr is None:
+        # All others convert to lowercase
+        return attr.lower()
+    return ret_attr
+
+
+def _process_info_installed_output(out, filter_attrs):
+    '''
+    Helper function for info_installed()
+
+    Processes stdout output from a single invocation of
+    'opkg status'.
+    '''
+    ret = {}
+    name = None
+    attrs = {}
+    attr = None
+
+    for line in salt.utils.itertools.split(out, '\n'):
+        if line and line[0] == ' ':
+            # This is a continuation of the last attr
+            if filter_attrs is None or attr in filter_attrs:
+                line = line.strip()
+                if len(attrs[attr]):
+                    # If attr is empty, don't add leading newline
+                    attrs[attr] += '\n'
+                attrs[attr] += line
+            continue
+        line = line.strip()
+        if not line:
+            # Separator between different packages
+            if name:
+                ret[name] = attrs
+            name = None
+            attrs = {}
+            attr = None
+            continue
+        key, value = line.split(':', 1)
+        value = value.lstrip()
+        attr = _convert_to_standard_attr(key)
+        if attr == 'name':
+            name = value
+        elif filter_attrs is None or attr in filter_attrs:
+            attrs[attr] = value
+
+    if name:
+        ret[name] = attrs
+    return ret
+
+
+def info_installed(*names, **kwargs):
+    '''
+    Return the information of the named package(s), installed on the system.
+
+    .. versionadded:: 2017.7.0
+
+    :param names:
+        Names of the packages to get information about. If none are specified,
+        will return information for all installed packages.
+
+    :param attr:
+        Comma-separated package attributes. If no 'attr' is specified, all available attributes returned.
+
+        Valid attributes are:
+            arch, conffiles, conflicts, depends, description, filename, group,
+            install_date_time_t, md5sum, packager, provides, recommends,
+            replaces, size, source, suggests, url, version
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.info_installed
+        salt '*' pkg.info_installed attr=version,packager
+        salt '*' pkg.info_installed <package1>
+        salt '*' pkg.info_installed <package1> <package2> <package3> ...
+        salt '*' pkg.info_installed <package1> attr=version,packager
+        salt '*' pkg.info_installed <package1> <package2> <package3> ... attr=version,packager
+    '''
+    attr = kwargs.pop('attr', None)
+    if attr is None:
+        filter_attrs = None
+    elif isinstance(attr, six.string_types):
+        filter_attrs = set(attr.split(','))
+    else:
+        filter_attrs = set(attr)
+
+    ret = {}
+    if names:
+        # Specific list of names of installed packages
+        for name in names:
+            cmd = ['opkg', 'status', name]
+            call = __salt__['cmd.run_all'](cmd,
+                                           output_loglevel='trace',
+                                           python_shell=False)
+            if call['retcode'] != 0:
+                comment = ''
+                if call['stderr']:
+                    comment += call['stderr']
+                else:
+                    comment += call['stdout']
+
+                raise CommandExecutionError(comment)
+            ret.update(_process_info_installed_output(call['stdout'], filter_attrs))
+    else:
+        # All installed packages
+        cmd = ['opkg', 'status']
+        call = __salt__['cmd.run_all'](cmd,
+                                       output_loglevel='trace',
+                                       python_shell=False)
+        if call['retcode'] != 0:
+            comment = ''
+            if call['stderr']:
+                comment += call['stderr']
+            else:
+                comment += call['stdout']
+
+            raise CommandExecutionError(comment)
+        ret.update(_process_info_installed_output(call['stdout'], filter_attrs))
+
+    return ret
+
+
+def upgrade_available(name, **kwargs):  # pylint: disable=unused-argument
     '''
     Check whether or not an upgrade is available for a given package
 
@@ -699,8 +1081,7 @@ def upgrade_available(name):
     return latest_version(name) != ''
 
 
-@_which('opkg-compare-versions')
-def version_cmp(pkg1, pkg2, ignore_epoch=False):
+def version_cmp(pkg1, pkg2, ignore_epoch=False, **kwargs):  # pylint: disable=unused-argument
     '''
     Do a cmp-style comparison on two packages. Return -1 if pkg1 < pkg2, 0 if
     pkg1 == pkg2, and 1 if pkg1 > pkg2. Return None if there was a problem
@@ -717,11 +1098,24 @@ def version_cmp(pkg1, pkg2, ignore_epoch=False):
 
         salt '*' pkg.version_cmp '0.2.4-0' '0.2.4.1-0'
     '''
-    normalize = lambda x: str(x).split(':', 1)[-1] if ignore_epoch else str(x)
+    normalize = lambda x: six.text_type(x).split(':', 1)[-1] if ignore_epoch else six.text_type(x)
     pkg1 = normalize(pkg1)
     pkg2 = normalize(pkg2)
 
-    cmd_compare = ['opkg-compare-versions']
+    output = __salt__['cmd.run_stdout'](['opkg', '--version'],
+                                        output_loglevel='trace',
+                                        python_shell=False)
+    opkg_version = output.split(' ')[2].strip()
+    if salt.utils.versions.LooseVersion(opkg_version) >= \
+            salt.utils.versions.LooseVersion('0.3.4'):
+        cmd_compare = ['opkg', 'compare-versions']
+    elif salt.utils.path.which('opkg-compare-versions'):
+        cmd_compare = ['opkg-compare-versions']
+    else:
+        log.warning('Unable to find a compare-versions utility installed. Either upgrade opkg to '
+                    'version > 0.3.4 (preferred) or install the older opkg-compare-versions script.')
+        return None
+
     for oper, ret in (("<<", -1), ("=", 0), (">>", 1)):
         cmd = cmd_compare[:]
         cmd.append(_cmd_quote(pkg1))
@@ -736,9 +1130,9 @@ def version_cmp(pkg1, pkg2, ignore_epoch=False):
     return None
 
 
-def list_repos():
+def list_repos(**kwargs):  # pylint: disable=unused-argument
     '''
-    Lists all repos on /etc/opkg/*.conf
+    Lists all repos on ``/etc/opkg/*.conf``
 
     CLI Example:
 
@@ -750,8 +1144,9 @@ def list_repos():
     regex = re.compile(REPO_REGEXP)
     for filename in os.listdir(OPKG_CONFDIR):
         if filename.endswith(".conf"):
-            with open(os.path.join(OPKG_CONFDIR, filename)) as conf_file:
+            with salt.utils.files.fopen(os.path.join(OPKG_CONFDIR, filename)) as conf_file:
                 for line in conf_file:
+                    line = salt.utils.stringutils.to_unicode(line)
                     if regex.search(line):
                         repo = {}
                         if line.startswith('#'):
@@ -759,7 +1154,7 @@ def list_repos():
                             line = line[1:]
                         else:
                             repo['enabled'] = True
-                        cols = line.strip().split()
+                        cols = salt.utils.args.shlex_split(line.strip())
                         if cols[0] in 'src':
                             repo['compressed'] = False
                         else:
@@ -773,9 +1168,9 @@ def list_repos():
     return repos
 
 
-def get_repo(alias):
+def get_repo(alias, **kwargs):  # pylint: disable=unused-argument
     '''
-    Display a repo from the /etc/opkg/*.conf
+    Display a repo from the ``/etc/opkg/*.conf``
 
     CLI Examples:
 
@@ -797,17 +1192,18 @@ def _del_repo_from_file(alias, filepath):
     '''
     Remove a repo from filepath
     '''
-    with open(filepath) as fhandle:
+    with salt.utils.files.fopen(filepath) as fhandle:
         output = []
         regex = re.compile(REPO_REGEXP)
         for line in fhandle:
+            line = salt.utils.stringutils.to_unicode(line)
             if regex.search(line):
                 if line.startswith('#'):
                     line = line[1:]
-                cols = line.strip().split()
+                cols = salt.utils.args.shlex_split(line.strip())
                 if alias != cols[1]:
-                    output.append(line)
-    with open(filepath, 'w') as fhandle:
+                    output.append(salt.utils.stringutils.to_str(line))
+    with salt.utils.files.fopen(filepath, 'w') as fhandle:
         fhandle.writelines(output)
 
 
@@ -817,31 +1213,38 @@ def _add_new_repo(alias, uri, compressed, enabled=True):
     '''
     repostr = '# ' if not enabled else ''
     repostr += 'src/gz ' if compressed else 'src '
-    repostr += alias + ' ' + uri + '\n'
+    if ' ' in alias:
+        repostr += '"' + alias + '" '
+    else:
+        repostr += alias + ' '
+    repostr += uri + '\n'
     conffile = os.path.join(OPKG_CONFDIR, alias + '.conf')
 
-    with open(conffile, 'a') as fhandle:
-        fhandle.write(repostr)
+    with salt.utils.files.fopen(conffile, 'a') as fhandle:
+        fhandle.write(salt.utils.stringutils.to_str(repostr))
 
 
 def _mod_repo_in_file(alias, repostr, filepath):
     '''
     Replace a repo entry in filepath with repostr
     '''
-    with open(filepath) as fhandle:
+    with salt.utils.files.fopen(filepath) as fhandle:
         output = []
         for line in fhandle:
-            if alias not in line:
+            cols = salt.utils.args.shlex_split(
+                salt.utils.stringutils.to_unicode(line).strip()
+            )
+            if alias not in cols:
                 output.append(line)
             else:
-                output.append(repostr + '\n')
-    with open(filepath, 'w') as fhandle:
+                output.append(salt.utils.stringutils.to_str(repostr + '\n'))
+    with salt.utils.files.fopen(filepath, 'w') as fhandle:
         fhandle.writelines(output)
 
 
-def del_repo(alias):
+def del_repo(alias, **kwargs):  # pylint: disable=unused-argument
     '''
-    Delete a repo from /etc/opkg/*.conf
+    Delete a repo from ``/etc/opkg/*.conf``
 
     If the file does not contain any other repo configuration, the file itself
     will be deleted.
@@ -927,7 +1330,11 @@ def mod_repo(alias, **kwargs):
                 repostr += 'src/gz ' if kwargs['compressed'] else 'src'
             else:
                 repostr += 'src/gz' if source['compressed'] else 'src'
-            repostr += ' {0}'.format(kwargs['alias'] if 'alias' in kwargs else alias)
+            repo_alias = kwargs['alias'] if 'alias' in kwargs else alias
+            if ' ' in repo_alias:
+                repostr += ' "{0}"'.format(repo_alias)
+            else:
+                repostr += ' {0}'.format(repo_alias)
             repostr += ' {0}'.format(kwargs['uri'] if 'uri' in kwargs else source['uri'])
             _mod_repo_in_file(alias, repostr, source['file'])
         elif uri and source['uri'] == uri:
@@ -949,7 +1356,7 @@ def mod_repo(alias, **kwargs):
         refresh_db()
 
 
-def file_list(*packages):
+def file_list(*packages, **kwargs):  # pylint: disable=unused-argument
     '''
     List the files that belong to a package. Not specifying any packages will
     return a list of _every_ file on the system's package database (not
@@ -970,7 +1377,7 @@ def file_list(*packages):
     return {'errors': output['errors'], 'files': files}
 
 
-def file_dict(*packages):
+def file_dict(*packages, **kwargs):  # pylint: disable=unused-argument
     '''
     List the files that belong to a package, grouped by package. Not
     specifying any packages will return a list of _every_ file on the system's
@@ -1012,7 +1419,7 @@ def file_dict(*packages):
     return {'errors': errors, 'packages': ret}
 
 
-def owner(*paths):
+def owner(*paths, **kwargs):  # pylint: disable=unused-argument
     '''
     Return the name of the package that owns the file. Multiple file paths can
     be passed. Like :mod:`pkg.version <salt.modules.opkg.version`, if a single
@@ -1042,5 +1449,5 @@ def owner(*paths):
         else:
             ret[path] = ''
     if len(ret) == 1:
-        return six.itervalues(ret)
+        return next(six.itervalues(ret))
     return ret
