@@ -2,9 +2,9 @@
 '''
 Module for managing timezone on POSIX-like systems.
 '''
-from __future__ import absolute_import
 
-# Import python libs
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import filecmp
 import os
 import errno
@@ -13,8 +13,12 @@ import re
 import string
 
 # Import salt libs
-import salt.utils
+import salt.utils.files
+import salt.utils.hashutils
 import salt.utils.itertools
+import salt.utils.path
+import salt.utils.platform
+import salt.utils.stringutils
 from salt.exceptions import SaltInvocationError, CommandExecutionError
 
 log = logging.getLogger(__name__)
@@ -26,12 +30,12 @@ def __virtual__():
     '''
     Only work on POSIX-like systems
     '''
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return (False, 'The timezone execution module failed to load: '
                        'win_timezone.py should replace this module on Windows.'
                        'There was a problem loading win_timezone.py.')
 
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return (False, 'The timezone execution module failed to load: '
                        'mac_timezone.py should replace this module on macOS.'
                        'There was a problem loading mac_timezone.py.')
@@ -54,8 +58,9 @@ def _timedatectl():
 
 def _get_zone_solaris():
     tzfile = '/etc/TIMEZONE'
-    with salt.utils.fopen(tzfile, 'r') as fp_:
+    with salt.utils.files.fopen(tzfile, 'r') as fp_:
         for line in fp_:
+            line = salt.utils.stringutils.to_unicode(line)
             if 'TZ=' in line:
                 zonepart = line.rstrip('\n').split('=')[-1]
                 return zonepart.strip('\'"') or 'UTC'
@@ -81,8 +86,9 @@ def _get_adjtime_timezone():
 
 def _get_zone_sysconfig():
     tzfile = '/etc/sysconfig/clock'
-    with salt.utils.fopen(tzfile, 'r') as fp_:
+    with salt.utils.files.fopen(tzfile, 'r') as fp_:
         for line in fp_:
+            line = salt.utils.stringutils.to_unicode(line)
             if re.match(r'^\s*#', line):
                 continue
             if 'ZONE' in line and '=' in line:
@@ -92,7 +98,7 @@ def _get_zone_sysconfig():
 
 
 def _get_zone_etc_localtime():
-    tzfile = '/etc/localtime'
+    tzfile = _get_localtime_path()
     tzdir = '/usr/share/zoneinfo/'
     tzdir_len = len(tzdir)
     try:
@@ -115,16 +121,16 @@ def _get_zone_etc_localtime():
             )
             # Regular file. Try to match the hash.
             hash_type = __opts__.get('hash_type', 'md5')
-            tzfile_hash = salt.utils.get_hash(tzfile, hash_type)
+            tzfile_hash = salt.utils.hashutils.get_hash(tzfile, hash_type)
             # Not a link, just a copy of the tzdata file
-            for root, dirs, files in os.walk(tzdir):
+            for root, dirs, files in salt.utils.path.os_walk(tzdir):
                 for filename in files:
                     full_path = os.path.join(root, filename)
                     olson_name = full_path[tzdir_len:]
                     if olson_name[0] in string.ascii_lowercase:
                         continue
                     if tzfile_hash == \
-                            salt.utils.get_hash(full_path, hash_type):
+                            salt.utils.hashutils.get_hash(full_path, hash_type):
                         return olson_name
     raise CommandExecutionError('Unable to determine timezone')
 
@@ -132,8 +138,8 @@ def _get_zone_etc_localtime():
 def _get_zone_etc_timezone():
     tzfile = '/etc/timezone'
     try:
-        with salt.utils.fopen(tzfile, 'r') as fp_:
-            return fp_.read().strip()
+        with salt.utils.files.fopen(tzfile, 'r') as fp_:
+            return salt.utils.stringutils.to_unicode(fp_.read()).strip()
     except IOError as exc:
         raise CommandExecutionError(
             'Problem reading timezone file {0}: {1}'
@@ -143,8 +149,9 @@ def _get_zone_etc_timezone():
 
 def _get_zone_aix():
     tzfile = '/etc/environment'
-    with salt.utils.fopen(tzfile, 'r') as fp_:
+    with salt.utils.files.fopen(tzfile, 'r') as fp_:
         for line in fp_:
+            line = salt.utils.stringutils.to_unicode(line)
             if 'TZ=' in line:
                 zonepart = line.rstrip('\n').split('=')[-1]
                 return zonepart.strip('\'"') or 'UTC'
@@ -168,7 +175,7 @@ def get_zone():
 
         salt '*' timezone.get_zone
     '''
-    if salt.utils.which('timedatectl'):
+    if salt.utils.path.which('timedatectl'):
         ret = _timedatectl()
 
         for line in (x.strip() for x in salt.utils.itertools.split(ret['stdout'], '\n')):
@@ -260,7 +267,7 @@ def set_zone(timezone):
         salt '*' timezone.set_zone 'CST6CDT,M3.2.0/2:00:00,M11.1.0/2:00:00'
 
     '''
-    if salt.utils.which('timedatectl'):
+    if salt.utils.path.which('timedatectl'):
         try:
             __salt__['cmd.run']('timedatectl set-timezone {0}'.format(timezone))
         except CommandExecutionError:
@@ -274,14 +281,15 @@ def set_zone(timezone):
     if not os.path.exists(zonepath) and 'AIX' not in __grains__['os_family']:
         return 'Zone does not exist: {0}'.format(zonepath)
 
-    if os.path.exists('/etc/localtime'):
-        os.unlink('/etc/localtime')
+    tzfile = _get_localtime_path()
+    if os.path.exists(tzfile):
+        os.unlink(tzfile)
 
     if 'Solaris' in __grains__['os_family']:
         __salt__['file.sed'](
             '/etc/default/init', '^TZ=.*', 'TZ={0}'.format(timezone))
     elif 'AIX' in __grains__['os_family']:
-        ## timezone could be Olson or Posix
+        # timezone could be Olson or Posix
         curtzstring = get_zone()
         cmd = ['chtz', timezone]
         result = __salt__['cmd.retcode'](cmd, python_shell=False)
@@ -293,7 +301,7 @@ def set_zone(timezone):
         __salt__['cmd.retcode'](cmd, python_shell=False)
         return False
     else:
-        os.symlink(zonepath, '/etc/localtime')
+        os.symlink(zonepath, tzfile)
 
     if 'RedHat' in __grains__['os_family']:
         __salt__['file.sed'](
@@ -302,8 +310,8 @@ def set_zone(timezone):
         __salt__['file.sed'](
             '/etc/sysconfig/clock', '^TIMEZONE=.*', 'TIMEZONE="{0}"'.format(timezone))
     elif 'Debian' in __grains__['os_family'] or 'Gentoo' in __grains__['os_family']:
-        with salt.utils.fopen('/etc/timezone', 'w') as ofh:
-            ofh.write(timezone.strip())
+        with salt.utils.files.fopen('/etc/timezone', 'w') as ofh:
+            ofh.write(salt.utils.stringutils.to_str(timezone).strip())
             ofh.write('\n')
 
     return True
@@ -337,7 +345,11 @@ def zone_compare(timezone):
     if 'Solaris' in __grains__['os_family'] or 'AIX' in __grains__['os_family']:
         return timezone == get_zone()
 
-    tzfile = _get_etc_localtime_path()
+    if 'FreeBSD' in __grains__['os_family']:
+        if not os.path.isfile(_get_localtime_path()):
+            return timezone == get_zone()
+
+    tzfile = _get_localtime_path()
     zonepath = _get_zone_file(timezone)
     try:
         return filecmp.cmp(tzfile, zonepath, shallow=False)
@@ -353,7 +365,9 @@ def zone_compare(timezone):
         raise
 
 
-def _get_etc_localtime_path():
+def _get_localtime_path():
+    if 'NILinuxRT' in __grains__['os_family'] and 'nilrt' in __grains__['lsb_distrib_id']:
+        return '/etc/natinst/share/localtime'
     return '/etc/localtime'
 
 
@@ -371,7 +385,7 @@ def get_hwclock():
 
         salt '*' timezone.get_hwclock
     '''
-    if salt.utils.which('timedatectl'):
+    if salt.utils.path.which('timedatectl'):
         ret = _timedatectl()
         for line in (x.strip() for x in ret['stdout'].splitlines()):
             if 'rtc in local tz' in line.lower():
@@ -389,15 +403,16 @@ def get_hwclock():
 
     else:
         os_family = __grains__['os_family']
-        for family in ('RedHat', 'Suse'):
+        for family in ('RedHat', 'Suse', 'NILinuxRT'):
             if family in os_family:
                 return _get_adjtime_timezone()
 
         if 'Debian' in __grains__['os_family']:
             # Original way to look up hwclock on Debian-based systems
             try:
-                with salt.utils.fopen('/etc/default/rcS', 'r') as fp_:
+                with salt.utils.files.fopen('/etc/default/rcS', 'r') as fp_:
                     for line in fp_:
+                        line = salt.utils.stringutils.to_unicode(line)
                         if re.match(r'^\s*#', line):
                             continue
                         if 'UTC=' in line:
@@ -415,8 +430,9 @@ def get_hwclock():
             if not os.path.exists('/etc/adjtime'):
                 offset_file = '/etc/conf.d/hwclock'
                 try:
-                    with salt.utils.fopen(offset_file, 'r') as fp_:
+                    with salt.utils.files.fopen(offset_file, 'r') as fp_:
                         for line in fp_:
+                            line = salt.utils.stringutils.to_unicode(line)
                             if line.startswith('clock='):
                                 line = line.rstrip('\n')
                                 line = line.split('=')[-1].strip('\'"')
@@ -438,8 +454,9 @@ def get_hwclock():
         if 'Solaris' in __grains__['os_family']:
             offset_file = '/etc/rtc_config'
             try:
-                with salt.utils.fopen(offset_file, 'r') as fp_:
+                with salt.utils.files.fopen(offset_file, 'r') as fp_:
                     for line in fp_:
+                        line = salt.utils.stringutils.to_unicode(line)
                         if line.startswith('zone_info=GMT'):
                             return 'UTC'
                     return 'localtime'
@@ -455,8 +472,9 @@ def get_hwclock():
         if 'AIX' in __grains__['os_family']:
             offset_file = '/etc/environment'
             try:
-                with salt.utils.fopen(offset_file, 'r') as fp_:
+                with salt.utils.files.fopen(offset_file, 'r') as fp_:
                     for line in fp_:
+                        line = salt.utils.stringutils.to_unicode(line)
                         if line.startswith('TZ=UTC'):
                             return 'UTC'
                     return 'localtime'
@@ -480,60 +498,66 @@ def set_hwclock(clock):
 
         salt '*' timezone.set_hwclock UTC
     '''
-    if 'AIX' in __grains__['os_family']:
-        if clock.lower() != 'utc':
-            raise SaltInvocationError(
-                'UTC is the only permitted value'
-            )
-        return True
-
-    timezone = get_zone()
-
-    if 'Solaris' in __grains__['os_family']:
-        if clock.lower() not in ('localtime', 'utc'):
-            raise SaltInvocationError(
-                'localtime and UTC are the only permitted values'
-            )
-        if 'sparc' in __grains__['cpuarch']:
-            raise SaltInvocationError(
-                'UTC is the only choice for SPARC architecture'
-            )
-        cmd = ['rtc', '-z', 'GMT' if clock.lower() == 'utc' else timezone]
-        return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
-
-    zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)
-
-    if not os.path.exists(zonepath):
-        raise CommandExecutionError(
-            'Zone \'{0}\' does not exist'.format(zonepath)
-        )
-
-    os.unlink('/etc/localtime')
-    os.symlink(zonepath, '/etc/localtime')
-
-    if 'Arch' in __grains__['os_family']:
-        cmd = ['timezonectl', 'set-local-rtc',
+    if salt.utils.path.which('timedatectl'):
+        cmd = ['timedatectl', 'set-local-rtc',
                'true' if clock == 'localtime' else 'false']
         return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
-    elif 'RedHat' in __grains__['os_family']:
-        __salt__['file.sed'](
-            '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
-    elif 'Suse' in __grains__['os_family']:
-        __salt__['file.sed'](
-            '/etc/sysconfig/clock', '^TIMEZONE=.*', 'TIMEZONE="{0}"'.format(timezone))
-    elif 'Debian' in __grains__['os_family']:
-        if clock == 'UTC':
-            __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=yes')
-        elif clock == 'localtime':
-            __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=no')
-    elif 'Gentoo' in __grains__['os_family']:
-        if clock not in ('UTC', 'localtime'):
-            raise SaltInvocationError(
-                'Only \'UTC\' and \'localtime\' are allowed'
+    else:
+        os_family = __grains__['os_family']
+        if os_family in ('AIX', 'NILinuxRT'):
+            if clock.lower() != 'utc':
+                raise SaltInvocationError(
+                    'UTC is the only permitted value'
+                )
+            return True
+
+        timezone = get_zone()
+
+        if 'Solaris' in __grains__['os_family']:
+            if clock.lower() not in ('localtime', 'utc'):
+                raise SaltInvocationError(
+                    'localtime and UTC are the only permitted values'
+                )
+            if 'sparc' in __grains__['cpuarch']:
+                raise SaltInvocationError(
+                    'UTC is the only choice for SPARC architecture'
+                )
+            cmd = ['rtc', '-z', 'GMT' if clock.lower() == 'utc' else timezone]
+            return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
+
+        zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)
+
+        if not os.path.exists(zonepath):
+            raise CommandExecutionError(
+                'Zone \'{0}\' does not exist'.format(zonepath)
             )
-        if clock == 'localtime':
-            clock = 'local'
-        __salt__['file.sed'](
-            '/etc/conf.d/hwclock', '^clock=.*', 'clock="{0}"'.format(clock))
+
+        os.unlink('/etc/localtime')
+        os.symlink(zonepath, '/etc/localtime')
+
+        if 'Arch' in __grains__['os_family']:
+            cmd = ['timezonectl', 'set-local-rtc',
+                   'true' if clock == 'localtime' else 'false']
+            return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
+        elif 'RedHat' in __grains__['os_family']:
+            __salt__['file.sed'](
+                '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
+        elif 'Suse' in __grains__['os_family']:
+            __salt__['file.sed'](
+                '/etc/sysconfig/clock', '^TIMEZONE=.*', 'TIMEZONE="{0}"'.format(timezone))
+        elif 'Debian' in __grains__['os_family']:
+            if clock == 'UTC':
+                __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=yes')
+            elif clock == 'localtime':
+                __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=no')
+        elif 'Gentoo' in __grains__['os_family']:
+            if clock not in ('UTC', 'localtime'):
+                raise SaltInvocationError(
+                    'Only \'UTC\' and \'localtime\' are allowed'
+                )
+            if clock == 'localtime':
+                clock = 'local'
+            __salt__['file.sed'](
+                '/etc/conf.d/hwclock', '^clock=.*', 'clock="{0}"'.format(clock))
 
     return True
