@@ -23,30 +23,31 @@ Example output::
                 - Hello
                 - World
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 # Import python libs
-import salt.utils.odict
 from numbers import Number
 
 # Import salt libs
 import salt.output
-from salt.ext.six import string_types
-from salt.utils import get_colors
-import salt.utils.locales
+import salt.utils.color
+import salt.utils.odict
+import salt.utils.stringutils
+from salt.ext import six
 
 
 class NestDisplay(object):
     '''
     Manage the nested display contents
     '''
-    def __init__(self):
+    def __init__(self, retcode=0):
         self.__dict__.update(
-            get_colors(
+            salt.utils.color.get_colors(
                 __opts__.get('color'),
                 __opts__.get('color_theme')
             )
         )
         self.strip_colors = __opts__.get('strip_colors', True)
+        self.retcode = retcode
 
     def ustring(self,
                 indent,
@@ -59,17 +60,46 @@ class NestDisplay(object):
             endc = self.ENDC
 
         indent *= ' '
-        fmt = u'{0}{1}{2}{3}{4}{5}'
+        fmt = '{0}{1}{2}{3}{4}{5}'
 
         try:
-            return fmt.format(indent, color, prefix, msg, endc, suffix)
+            return fmt.format(
+                indent,
+                color,
+                prefix,
+                msg,
+                endc,
+                suffix)
         except UnicodeDecodeError:
-            return fmt.format(indent, color, prefix, salt.utils.locales.sdecode(msg), endc, suffix)
+            try:
+                return fmt.format(
+                    indent,
+                    color,
+                    prefix,
+                    salt.utils.stringutils.to_unicode(msg),
+                    endc,
+                    suffix)
+            except UnicodeDecodeError:
+                # msg contains binary data that can't be decoded
+                return str(fmt).format(  # future lint: disable=blacklisted-function
+                    indent,
+                    color,
+                    prefix,
+                    msg,
+                    endc,
+                    suffix)
 
     def display(self, ret, indent, prefix, out):
         '''
         Recursively iterate down through data structures to determine output
         '''
+        if isinstance(ret, bytes):
+            try:
+                ret = salt.utils.stringutils.to_unicode(ret)
+            except UnicodeDecodeError:
+                # ret contains binary data that can't be decoded
+                pass
+
         if ret is None or ret is True or ret is False:
             out.append(
                 self.ustring(
@@ -81,16 +111,18 @@ class NestDisplay(object):
             )
         # Number includes all python numbers types
         #  (float, int, long, complex, ...)
+        # use repr() to get the full precision also for older python versions
+        # as until about python32 it was limited to 12 digits only by default
         elif isinstance(ret, Number):
             out.append(
                 self.ustring(
                     indent,
                     self.LIGHT_YELLOW,
-                    ret,
+                    repr(ret),
                     prefix=prefix
                 )
             )
-        elif isinstance(ret, string_types):
+        elif isinstance(ret, six.string_types):
             first_line = True
             for line in ret.splitlines():
                 if self.strip_colors:
@@ -106,12 +138,15 @@ class NestDisplay(object):
                 )
                 first_line = False
         elif isinstance(ret, (list, tuple)):
+            color = self.GREEN
+            if self.retcode != 0:
+                color = self.RED
             for ind in ret:
                 if isinstance(ind, (list, tuple, dict)):
                     out.append(
                         self.ustring(
                             indent,
-                            self.GREEN,
+                            color,
                             '|_'
                         )
                     )
@@ -121,10 +156,13 @@ class NestDisplay(object):
                     self.display(ind, indent, '- ', out)
         elif isinstance(ret, dict):
             if indent:
+                color = self.CYAN
+                if self.retcode != 0:
+                    color = self.RED
                 out.append(
                     self.ustring(
                         indent,
-                        self.CYAN,
+                        color,
                         '----------'
                     )
                 )
@@ -134,13 +172,15 @@ class NestDisplay(object):
                 keys = ret.keys()
             else:
                 keys = sorted(ret)
-
+            color = self.CYAN
+            if self.retcode != 0:
+                color = self.RED
             for key in keys:
                 val = ret[key]
                 out.append(
                     self.ustring(
                         indent,
-                        self.CYAN,
+                        color,
                         key,
                         suffix=':',
                         prefix=prefix
@@ -155,7 +195,15 @@ def output(ret, **kwargs):
     Display ret data
     '''
     # Prefer kwargs before opts
+    retcode = kwargs.get('_retcode', 0)
     base_indent = kwargs.get('nested_indent', 0) \
         or __opts__.get('nested_indent', 0)
-    nest = NestDisplay()
-    return '\n'.join(nest.display(ret, base_indent, '', []))
+    nest = NestDisplay(retcode=retcode)
+    lines = nest.display(ret, base_indent, '', [])
+    try:
+        return '\n'.join(lines)
+    except UnicodeDecodeError:
+        # output contains binary data that can't be decoded
+        return str('\n').join(  # future lint: disable=blacklisted-function
+            [salt.utils.stringutils.to_str(x) for x in lines]
+        )
