@@ -15,7 +15,6 @@ import salt.log
 import salt.utils.args
 import salt.utils.gitfs
 import salt.utils.master
-import salt.utils.versions
 import salt.payload
 import salt.cache
 import salt.fileserver.gitfs
@@ -37,22 +36,35 @@ def grains(tgt=None, tgt_type='glob', **kwargs):
         The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
         releases must use ``expr_form``.
 
-    Return cached grains of the targeted minions
+    Return cached grains of the targeted minions.
+
+    tgt
+        Target to match minion ids.
+
+        .. versionchanged:: 2017.7.5,2018.3.0
+            The ``tgt`` argument is now required to display cached grains. If
+            not used, the function will not return grains. This optional
+            argument will become mandatory in the Salt ``Sodium`` release.
+
+    tgt_type
+        The type of targeting to use for matching, such as ``glob``, ``list``,
+        etc.
 
     CLI Example:
 
     .. code-block:: bash
 
-        salt-run cache.grains
+        salt-run cache.grains '*'
     '''
-    if 'expr_form' in kwargs:
+    if tgt is None:
+        # Change ``tgt=None`` to ``tgt`` (mandatory kwarg) in Salt Sodium.
+        # This behavior was changed in PR #45588 to fix Issue #45489.
         salt.utils.versions.warn_until(
-            'Fluorine',
-            'The target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
+            'Sodium',
+            'Detected missing \'tgt\' option. Cached grains will not be returned '
+            'without a specified \'tgt\'. This option will be required starting in '
+            'Salt Sodium and this warning will be removed.'
         )
-        tgt_type = kwargs.pop('expr_form')
 
     pillar_util = salt.utils.master.MasterPillarUtil(tgt, tgt_type,
                                                      use_cached_grains=True,
@@ -76,15 +88,6 @@ def pillar(tgt=None, tgt_type='glob', **kwargs):
 
         salt-run cache.pillar
     '''
-    if 'expr_form' in kwargs:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'The target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = kwargs.pop('expr_form')
-
     pillar_util = salt.utils.master.MasterPillarUtil(tgt, tgt_type,
                                                      use_cached_grains=True,
                                                      grains_fallback=False,
@@ -109,15 +112,6 @@ def mine(tgt=None, tgt_type='glob', **kwargs):
 
         salt-run cache.mine
     '''
-    if 'expr_form' in kwargs:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'The target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = kwargs.pop('expr_form')
-
     pillar_util = salt.utils.master.MasterPillarUtil(tgt, tgt_type,
                                                      use_cached_grains=False,
                                                      grains_fallback=False,
@@ -151,7 +145,7 @@ def _clear_cache(tgt=None,
                                                 clear_mine_func=clear_mine_func_flag)
 
 
-def clear_pillar(tgt=None, tgt_type='glob', expr_form=None):
+def clear_pillar(tgt=None, tgt_type='glob'):
     '''
     .. versionchanged:: 2017.7.0
         The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
@@ -165,21 +159,10 @@ def clear_pillar(tgt=None, tgt_type='glob', expr_form=None):
 
         salt-run cache.clear_pillar
     '''
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     return _clear_cache(tgt, tgt_type, clear_pillar_flag=True)
 
 
-def clear_grains(tgt=None, tgt_type='glob', expr_form=None):
+def clear_grains(tgt=None, tgt_type='glob'):
     '''
     .. versionchanged:: 2017.7.0
         The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
@@ -193,21 +176,10 @@ def clear_grains(tgt=None, tgt_type='glob', expr_form=None):
 
         salt-run cache.clear_grains
     '''
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     return _clear_cache(tgt, tgt_type, clear_grains_flag=True)
 
 
-def clear_mine(tgt=None, tgt_type='glob', expr_form=None):
+def clear_mine(tgt=None, tgt_type='glob'):
     '''
     .. versionchanged:: 2017.7.0
         The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
@@ -221,24 +193,12 @@ def clear_mine(tgt=None, tgt_type='glob', expr_form=None):
 
         salt-run cache.clear_mine
     '''
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     return _clear_cache(tgt, tgt_type, clear_mine_flag=True)
 
 
 def clear_mine_func(tgt=None,
                     tgt_type='glob',
-                    clear_mine_func_flag=None,
-                    expr_form=None):
+                    clear_mine_func_flag=None):
     '''
     .. versionchanged:: 2017.7.0
         The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
@@ -255,7 +215,7 @@ def clear_mine_func(tgt=None,
     return _clear_cache(tgt, tgt_type, clear_mine_func_flag=clear_mine_func_flag)
 
 
-def clear_all(tgt=None, tgt_type='glob', expr_form=None):
+def clear_all(tgt=None, tgt_type='glob'):
     '''
     .. versionchanged:: 2017.7.0
         The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
@@ -269,17 +229,6 @@ def clear_all(tgt=None, tgt_type='glob', expr_form=None):
 
         salt-run cache.clear_all
     '''
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     return _clear_cache(tgt,
                         tgt_type,
                         clear_pillar_flag=True,
