@@ -21,19 +21,22 @@ modules. These state functions wrap Salt's :ref:`Python API <python-api>`.
     * :ref:`Full Orchestrate Tutorial <orchestrate-runner>`
     * :py:func:`The Orchestrate runner <salt.runners.state.orchestrate>`
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import python libs
 import fnmatch
 import logging
+import sys
+import threading
 import time
 
 # Import salt libs
 import salt.syspaths
-import salt.utils
+import salt.exceptions
+import salt.output
+import salt.utils.data
 import salt.utils.event
-import salt.ext.six as six
-from salt.ext.six import string_types
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -60,17 +63,60 @@ def _fire_args(tag_data):
         )
 
 
+def _parallel_map(func, inputs):
+    '''
+    Applies a function to each element of a list, returning the resulting list.
+
+    A separate thread is created for each element in the input list and the
+    passed function is called for each of the elements. When all threads have
+    finished execution a list with the results corresponding to the inputs is
+    returned.
+
+    If one of the threads fails (because the function throws an exception),
+    that exception is reraised. If more than one thread fails, the exception
+    from the first thread (according to the index of the input element) is
+    reraised.
+
+    func:
+        function that is applied on each input element.
+    inputs:
+        list of elements that shall be processed. The length of this list also
+        defines the number of threads created.
+    '''
+    outputs = len(inputs) * [None]
+    errors = len(inputs) * [None]
+
+    def create_thread(index):
+        def run_thread():
+            try:
+                outputs[index] = func(inputs[index])
+            except:  # pylint: disable=bare-except
+                errors[index] = sys.exc_info()
+        thread = threading.Thread(target=run_thread)
+        thread.start()
+        return thread
+    threads = list(six.moves.map(create_thread, six.moves.range(len(inputs))))
+    for thread in threads:
+        thread.join()
+    for error in errors:
+        if error is not None:
+            exc_type, exc_value, exc_traceback = error
+            six.reraise(exc_type, exc_value, exc_traceback)
+    return outputs
+
+
 def state(name,
         tgt,
         ssh=False,
         tgt_type='glob',
-        expr_form=None,
         ret='',
+        ret_config=None,
+        ret_kwargs=None,
         highstate=None,
         sls=None,
         top=None,
         saltenv=None,
-        test=False,
+        test=None,
         pillar=None,
         pillarenv=None,
         expect_minions=True,
@@ -81,7 +127,8 @@ def state(name,
         batch=None,
         queue=False,
         subset=None,
-        orchestration_jid=None):
+        orchestration_jid=None,
+        **kwargs):
     '''
     Invoke a state run on a given target
 
@@ -99,13 +146,15 @@ def state(name,
     tgt_type
         The target type to resolve, defaults to ``glob``
 
-    expr_form
-        .. deprecated:: 2017.7.0
-            Use tgt_type instead
-
     ret
         Optionally set a single or a list of returners to use
 
+    ret_config
+        Use an alternative returner configuration
+
+    ret_kwargs
+        Override individual returner configuration items
+
     highstate
         Defaults to None, if set to True the target systems will ignore any
         sls references specified in the sls option and call state.highstate
@@ -120,7 +169,10 @@ def state(name,
         containing a single sls file, or a list of sls files
 
     test
-        Pass ``test=true`` through to the state function
+        Pass ``test=true`` or ``test=false`` through to the state function. This
+        can be used to overide a test mode set in the minion's config file. If
+        left as the default of None and the 'test' mode is supplied on the
+        command line, that value is passed instead.
 
     pillar
         Pass the ``pillar`` kwarg through to the state function
@@ -198,6 +250,12 @@ def state(name,
     '''
     cmd_kw = {'arg': [], 'kwarg': {}, 'ret': ret, 'timeout': timeout}
 
+    if ret_config:
+        cmd_kw['ret_config'] = ret_config
+
+    if ret_kwargs:
+        cmd_kw['ret_kwargs'] = ret_kwargs
+
     state_ret = {'name': name,
                  'changes': {},
                  'comment': '',
@@ -210,17 +268,6 @@ def state(name,
         state_ret['comment'] = 'Passed invalid value for \'allow_fail\', must be an int'
         return state_ret
 
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     cmd_kw['tgt_type'] = tgt_type
     cmd_kw['ssh'] = ssh
     cmd_kw['expect_minions'] = expect_minions
@@ -239,8 +286,8 @@ def state(name,
         state_ret['result'] = False
         return state_ret
 
-    if test or __opts__.get('test'):
-        cmd_kw['kwarg']['test'] = True
+    if test is not None or __opts__.get('test'):
+        cmd_kw['kwarg']['test'] = test if test is not None else __opts__.get('test')
 
     if pillar:
         cmd_kw['kwarg']['pillar'] = pillar
@@ -261,7 +308,7 @@ def state(name,
         return state_ret
 
     if batch is not None:
-        cmd_kw['batch'] = str(batch)
+        cmd_kw['batch'] = six.text_type(batch)
     if subset is not None:
         cmd_kw['subset'] = subset
 
@@ -274,7 +321,7 @@ def state(name,
         if top:
             cmd_kw['topfn'] = ''.join(cmd_kw.pop('arg'))
         elif sls:
-            cmd_kw['mods'] = cmd_kw.pop('arg')
+            cmd_kw['mods'] = ''.join(cmd_kw.pop('arg'))
         cmd_kw.update(cmd_kw.pop('kwarg'))
         tmp_ret = __salt__[fun](**cmd_kw)
         cmd_ret = {__opts__['id']: {
@@ -290,12 +337,11 @@ def state(name,
 
     changes = {}
     fail = set()
-    failures = {}
     no_change = set()
 
     if fail_minions is None:
         fail_minions = ()
-    elif isinstance(fail_minions, string_types):
+    elif isinstance(fail_minions, six.string_types):
         fail_minions = [minion.strip() for minion in fail_minions.split(',')]
     elif not isinstance(fail_minions, list):
         state_ret.setdefault('warnings', []).append(
@@ -327,12 +373,12 @@ def state(name,
             except KeyError:
                 m_state = False
             if m_state:
-                m_state = salt.utils.check_state_result(m_ret, recurse=True)
+                m_state = __utils__['state.check_result'](m_ret, recurse=True)
 
         if not m_state:
             if minion not in fail_minions:
                 fail.add(minion)
-            failures[minion] = m_ret or 'Minion did not respond'
+            changes[minion] = m_ret
             continue
         try:
             for state_item in six.itervalues(m_ret):
@@ -357,18 +403,6 @@ def state(name,
             state_ret['comment'] += ' Updating {0}.'.format(', '.join(changes))
         if no_change:
             state_ret['comment'] += ' No changes made to {0}.'.format(', '.join(no_change))
-    if failures:
-        state_ret['comment'] += '\nFailures:\n'
-        for minion, failure in six.iteritems(failures):
-            state_ret['comment'] += '\n'.join(
-                    (' ' * 4 + l)
-                    for l in salt.output.out_format(
-                        {minion: failure},
-                        'highstate',
-                        __opts__,
-                        ).splitlines()
-                    )
-            state_ret['comment'] += '\n'
     if test or __opts__.get('test'):
         if state_ret['changes'] and state_ret['result'] is True:
             # Test mode with changes is the only case where result should ever be none
@@ -381,8 +415,9 @@ def function(
         tgt,
         ssh=False,
         tgt_type='glob',
-        expr_form=None,
         ret='',
+        ret_config=None,
+        ret_kwargs=None,
         expect_minions=False,
         fail_minions=None,
         fail_function=None,
@@ -403,10 +438,6 @@ def function(
     tgt_type
         The target type, defaults to ``glob``
 
-    expr_form
-        .. deprecated:: 2017.7.0
-            Use tgt_type instead
-
     arg
         The list of arguments to pass into the function
 
@@ -416,6 +447,12 @@ def function(
     ret
         Optionally set a single or a list of returners to use
 
+    ret_config
+        Use an alternative returner configuration
+
+    ret_kwargs
+        Override individual returner configuration items
+
     expect_minions
         An optional boolean for failing if some minions do not respond
 
@@ -444,7 +481,7 @@ def function(
            'result': True}
     if kwarg is None:
         kwarg = {}
-    if isinstance(arg, str):
+    if isinstance(arg, six.string_types):
         func_ret['warnings'] = ['Please specify \'arg\' as a list, not a string. '
                            'Modifying in place, but please update SLS file '
                            'to remove this warning.']
@@ -452,19 +489,8 @@ def function(
 
     cmd_kw = {'arg': arg or [], 'kwarg': kwarg, 'ret': ret, 'timeout': timeout}
 
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     if batch is not None:
-        cmd_kw['batch'] = str(batch)
+        cmd_kw['batch'] = six.text_type(batch)
     if subset is not None:
         cmd_kw['subset'] = subset
 
@@ -472,11 +498,18 @@ def function(
     cmd_kw['ssh'] = ssh
     cmd_kw['expect_minions'] = expect_minions
     cmd_kw['_cmd_meta'] = True
+
+    if ret_config:
+        cmd_kw['ret_config'] = ret_config
+
+    if ret_kwargs:
+        cmd_kw['ret_kwargs'] = ret_kwargs
+
     fun = name
     if __opts__['test'] is True:
         func_ret['comment'] = (
                 'Function {0} will be executed on target {1} as test={2}'
-                ).format(fun, tgt, str(False))
+                ).format(fun, tgt, six.text_type(False))
         func_ret['result'] = None
         return func_ret
     try:
@@ -484,7 +517,7 @@ def function(
         cmd_ret = __salt__['saltutil.cmd'](tgt, fun, **cmd_kw)
     except Exception as exc:
         func_ret['result'] = False
-        func_ret['comment'] = str(exc)
+        func_ret['comment'] = six.text_type(exc)
         return func_ret
 
     try:
@@ -494,11 +527,10 @@ def function(
 
     changes = {}
     fail = set()
-    failures = {}
 
     if fail_minions is None:
         fail_minions = ()
-    elif isinstance(fail_minions, string_types):
+    elif isinstance(fail_minions, six.string_types):
         fail_minions = [minion.strip() for minion in fail_minions.split(',')]
     elif not isinstance(fail_minions, list):
         func_ret.setdefault('warnings', []).append(
@@ -519,11 +551,12 @@ def function(
             m_ret = mdata['ret']
             m_func = (not fail_function and True) or __salt__[fail_function](m_ret)
 
+            if m_ret is False:
+                m_func = False
+
         if not m_func:
             if minion not in fail_minions:
                 fail.add(minion)
-            failures[minion] = m_ret and m_ret or 'Minion did not respond'
-            continue
         changes[minion] = m_ret
     if not cmd_ret:
         func_ret['result'] = False
@@ -538,18 +571,6 @@ def function(
             func_ret['comment'] = 'Function ran successfully.'
         if changes:
             func_ret['comment'] += ' Function {0} ran on {1}.'.format(name, ', '.join(changes))
-        if failures:
-            func_ret['comment'] += '\nFailures:\n'
-            for minion, failure in six.iteritems(failures):
-                func_ret['comment'] += '\n'.join(
-                        (' ' * 4 + l)
-                        for l in salt.output.out_format(
-                            {minion: failure},
-                            'highstate',
-                            __opts__,
-                            ).splitlines()
-                        )
-                func_ret['comment'] += '\n'
     return func_ret
 
 
@@ -639,23 +660,23 @@ def wait_for_event(
                 try:
                     val_idx = id_list.index(val)
                 except ValueError:
-                    log.trace("wait_for_event: Event identifier '{0}' not in "
-                            "id_list; skipping.".format(event_id))
+                    log.trace("wait_for_event: Event identifier '%s' not in "
+                              "id_list; skipping.", event_id)
                 else:
                     del id_list[val_idx]
                     del_counter += 1
                     minions_seen = ret['changes'].setdefault('minions_seen', [])
                     minions_seen.append(val)
 
-                    log.debug("wait_for_event: Event identifier '{0}' removed "
-                            "from id_list; {1} items remaining."
-                            .format(val, len(id_list)))
+                    log.debug("wait_for_event: Event identifier '%s' removed "
+                              "from id_list; %s items remaining.",
+                              val, len(id_list))
             else:
-                log.trace("wait_for_event: Event identifier '{0}' not in event "
-                        "'{1}'; skipping.".format(event_id, event['tag']))
+                log.trace("wait_for_event: Event identifier '%s' not in event "
+                          "'%s'; skipping.", event_id, event['tag'])
         else:
-            log.debug("wait_for_event: Skipping unmatched event '{0}'"
-                    .format(event['tag']))
+            log.debug("wait_for_event: Skipping unmatched event '%s'",
+                      event['tag'])
 
         if len(id_list) == 0:
             ret['result'] = True
@@ -711,20 +732,15 @@ def runner(name, **kwargs):
     runner_return = out.get('return')
     if isinstance(runner_return, dict) and 'Error' in runner_return:
         out['success'] = False
-    if not out.get('success', True):
-        ret = {
-            'name': name,
-            'result': False,
-            'changes': {},
-            'comment': runner_return if runner_return else "Runner function '{0}' failed without comment.".format(name)
-        }
-    else:
-        ret = {
-            'name': name,
-            'result': True,
-            'changes': runner_return if runner_return else {},
-            'comment': "Runner function '{0}' executed.".format(name)
-        }
+
+    success = out.get('success', True)
+    ret = {'name': name,
+           'changes': {'return': runner_return},
+           'result': success}
+    ret['comment'] = "Runner function '{0}' {1}.".format(
+        name,
+        'executed' if success else 'failed',
+    )
 
     ret['__orchestration__'] = True
     if 'jid' in out:
@@ -733,6 +749,190 @@ def runner(name, **kwargs):
     return ret
 
 
+def parallel_runners(name, runners):
+    '''
+    Executes multiple runner modules on the master in parallel.
+
+    .. versionadded:: 2017.x.0 (Nitrogen)
+
+    A separate thread is spawned for each runner. This state is intended to be
+    used with the orchestrate runner in place of the ``saltmod.runner`` state
+    when different tasks should be run in parallel. In general, Salt states are
+    not safe when used concurrently, so ensure that they are used in a safe way
+    (e.g. by only targeting separate minions in parallel tasks).
+
+    name:
+        name identifying this state. The name is provided as part of the
+        output, but not used for anything else.
+
+    runners:
+        list of runners that should be run in parallel. Each element of the
+        list has to be a dictionary. This dictionary's name entry stores the
+        name of the runner function that shall be invoked. The optional kwarg
+        entry stores a dictionary of named arguments that are passed to the
+        runner function.
+
+    .. code-block:: yaml
+
+        parallel-state:
+           salt.parallel_runners:
+             - runners:
+                 my_runner_1:
+                   - name: state.orchestrate
+                   - kwarg:
+                       mods: orchestrate_state_1
+                 my_runner_2:
+                   - name: state.orchestrate
+                   - kwarg:
+                       mods: orchestrate_state_2
+    '''
+    # For the sake of consistency, we treat a single string in the same way as
+    # a key without a value. This allows something like
+    #     salt.parallel_runners:
+    #       - runners:
+    #           state.orchestrate
+    # Obviously, this will only work if the specified runner does not need any
+    # arguments.
+    if isinstance(runners, six.string_types):
+        runners = {runners: [{name: runners}]}
+    # If the runners argument is not a string, it must be a dict. Everything
+    # else is considered an error.
+    if not isinstance(runners, dict):
+        return {
+            'name': name,
+            'result': False,
+            'changes': {},
+            'comment': 'The runners parameter must be a string or dict.'
+        }
+    # The configuration for each runner is given as a list of key-value pairs.
+    # This is not very useful for what we want to do, but it is the typical
+    # style used in Salt. For further processing, we convert each of these
+    # lists to a dict. This also makes it easier to check whether a name has
+    # been specified explicitly.
+    for runner_id, runner_config in six.iteritems(runners):
+        if runner_config is None:
+            runner_config = {}
+        else:
+            runner_config = salt.utils.data.repack_dictlist(runner_config)
+        if 'name' not in runner_config:
+            runner_config['name'] = runner_id
+        runners[runner_id] = runner_config
+
+    try:
+        jid = __orchestration_jid__
+    except NameError:
+        log.debug(
+            'Unable to fire args event due to missing __orchestration_jid__')
+        jid = None
+
+    def call_runner(runner_config):
+        return __salt__['saltutil.runner'](runner_config['name'],
+                                           __orchestration_jid__=jid,
+                                           __env__=__env__,
+                                           full_return=True,
+                                           **(runner_config.get('kwarg', {})))
+
+    try:
+        outputs = _parallel_map(call_runner, list(six.itervalues(runners)))
+    except salt.exceptions.SaltException as exc:
+        return {
+            'name': name,
+            'result': False,
+            'success': False,
+            'changes': {},
+            'comment': 'One of the runners raised an exception: {0}'.format(
+                exc)
+        }
+    # We bundle the results of the runners with the IDs of the runners so that
+    # we can easily identify which output belongs to which runner. At the same
+    # time we exctract the actual return value of the runner (saltutil.runner
+    # adds some extra information that is not interesting to us).
+    outputs = {
+        runner_id: out['return']for runner_id, out in
+        six.moves.zip(six.iterkeys(runners), outputs)
+    }
+
+    # If each of the runners returned its output in the format compatible with
+    # the 'highstate' outputter, we can leverage this fact when merging the
+    # outputs.
+    highstate_output = all(
+        [out.get('outputter', '') == 'highstate' and 'data' in out for out in
+         six.itervalues(outputs)]
+    )
+
+    # The following helper function is used to extract changes from highstate
+    # output.
+
+    def extract_changes(obj):
+        if not isinstance(obj, dict):
+            return {}
+        elif 'changes' in obj:
+            if (isinstance(obj['changes'], dict)
+                    and obj['changes'].get('out', '') == 'highstate'
+                    and 'ret' in obj['changes']):
+                return obj['changes']['ret']
+            else:
+                return obj['changes']
+        else:
+            found_changes = {}
+            for key, value in six.iteritems(obj):
+                change = extract_changes(value)
+                if change:
+                    found_changes[key] = change
+            return found_changes
+    if highstate_output:
+        failed_runners = [runner_id for runner_id, out in
+                          six.iteritems(outputs) if
+                          out['data'].get('retcode', 0) != 0]
+        all_successful = not failed_runners
+        if all_successful:
+            comment = 'All runner functions executed successfully.'
+        else:
+            runner_comments = [
+                'Runner {0} failed with return value:\n{1}'.format(
+                    runner_id,
+                    salt.output.out_format(outputs[runner_id],
+                                           'nested',
+                                           __opts__,
+                                           nested_indent=2)
+                ) for runner_id in failed_runners
+            ]
+            comment = '\n'.join(runner_comments)
+        changes = {}
+        for runner_id, out in six.iteritems(outputs):
+            runner_changes = extract_changes(out['data'])
+            if runner_changes:
+                changes[runner_id] = runner_changes
+    else:
+        failed_runners = [runner_id for runner_id, out in
+                          six.iteritems(outputs) if
+                          out.get('exit_code', 0) != 0]
+        all_successful = not failed_runners
+        if all_successful:
+            comment = 'All runner functions executed successfully.'
+        else:
+            if len(failed_runners) == 1:
+                comment = 'Runner {0} failed.'.format(failed_runners[0])
+            else:
+                comment =\
+                    'Runners {0} failed.'.format(', '.join(failed_runners))
+        changes = {'ret': {
+            runner_id: out for runner_id, out in six.iteritems(outputs)
+        }}
+    ret = {
+        'name': name,
+        'result': all_successful,
+        'changes': changes,
+        'comment': comment
+    }
+
+    # The 'runner' function includes out['jid'] as '__jid__' in the returned
+    # dict, but we cannot do this here because we have more than one JID if
+    # we have more than one runner.
+
+    return ret
+
+
 def wheel(name, **kwargs):
     '''
     Execute a wheel module on the master
@@ -771,15 +971,21 @@ def wheel(name, **kwargs):
                                      __env__=__env__,
                                      **kwargs)
 
-    ret['result'] = True
-    ret['comment'] = "Wheel function '{0}' executed.".format(name)
+    wheel_return = out.get('return')
+    if isinstance(wheel_return, dict) and 'Error' in wheel_return:
+        out['success'] = False
+
+    success = out.get('success', True)
+    ret = {'name': name,
+           'changes': {'return': wheel_return},
+           'result': success}
+    ret['comment'] = "Wheel function '{0}' {1}.".format(
+        name,
+        'executed' if success else 'failed',
+    )
 
     ret['__orchestration__'] = True
     if 'jid' in out:
         ret['__jid__'] = out['jid']
 
-    runner_return = out.get('return')
-    if runner_return:
-        ret['changes'] = runner_return
-
     return ret
