@@ -8,19 +8,20 @@ Wire protocol: "len(payload) msgpack({'head': SOMEHEADER, 'body': SOMEBODY})"
 
 # Import Python Libs
 from __future__ import absolute_import, print_function, unicode_literals
+import errno
 import logging
-import msgpack
 import socket
 import os
 import weakref
 import time
+import threading
 import traceback
-import errno
 
 # Import Salt Libs
 import salt.crypt
-import salt.utils.async
+import salt.utils.asynchronous
 import salt.utils.event
+import salt.utils.files
 import salt.utils.platform
 import salt.utils.process
 import salt.utils.verify
@@ -32,6 +33,7 @@ import salt.transport.client
 import salt.transport.server
 import salt.transport.mixins.auth
 from salt.ext import six
+from salt.ext.six.moves import queue  # pylint: disable=import-error
 from salt.exceptions import SaltReqTimeoutError, SaltClientError
 from salt.transport import iter_transport_opts
 
@@ -51,6 +53,7 @@ else:
 # pylint: enable=import-error,no-name-in-module
 
 # Import third party libs
+import msgpack
 try:
     from M2Crypto import RSA
     HAS_M2 = True
@@ -69,7 +72,6 @@ else:
 if USE_LOAD_BALANCER:
     import threading
     import multiprocessing
-    import errno
     import tornado.util
     from salt.utils.process import SignalHandlingMultiprocessingProcess
 
@@ -144,8 +146,8 @@ if USE_LOAD_BALANCER:
         # Based on default used in tornado.netutil.bind_sockets()
         backlog = 128
 
-        def __init__(self, opts, socket_queue, log_queue=None):
-            super(LoadBalancerServer, self).__init__(log_queue=log_queue)
+        def __init__(self, opts, socket_queue, **kwargs):
+            super(LoadBalancerServer, self).__init__(**kwargs)
             self.opts = opts
             self.socket_queue = socket_queue
             self._socket = None
@@ -159,13 +161,17 @@ if USE_LOAD_BALANCER:
             self.__init__(
                 state['opts'],
                 state['socket_queue'],
-                log_queue=state['log_queue']
+                log_queue=state['log_queue'],
+                log_queue_level=state['log_queue_level']
             )
 
         def __getstate__(self):
-            return {'opts': self.opts,
-                    'socket_queue': self.socket_queue,
-                    'log_queue': self.log_queue}
+            return {
+                'opts': self.opts,
+                'socket_queue': self.socket_queue,
+                'log_queue': self.log_queue,
+                'log_queue_level': self.log_queue_level
+            }
 
         def close(self):
             if self._socket is not None:
@@ -235,8 +241,13 @@ class AsyncTCPReqChannel(salt.transport.client.ReqChannel):
             # references it-- this forces a reference while we return to the caller
             obj = object.__new__(cls)
             obj.__singleton_init__(opts, **kwargs)
+            obj._instance_key = key
             loop_instance_map[key] = obj
+            obj._refcount = 1
+            obj._refcount_lock = threading.RLock()
         else:
+            with obj._refcount_lock:
+                obj._refcount += 1
             log.debug('Re-using AsyncTCPReqChannel for %s', key)
         return obj
 
@@ -283,11 +294,43 @@ class AsyncTCPReqChannel(salt.transport.client.ReqChannel):
     def close(self):
         if self._closing:
             return
+
+        if self._refcount > 1:
+            # Decrease refcount
+            with self._refcount_lock:
+                self._refcount -= 1
+            log.debug(
+                'This is not the last %s instance. Not closing yet.',
+                self.__class__.__name__
+            )
+            return
+
+        log.debug('Closing %s instance', self.__class__.__name__)
         self._closing = True
         self.message_client.close()
 
+        # Remove the entry from the instance map so that a closed entry may not
+        # be reused.
+        # This forces this operation even if the reference count of the entry
+        # has not yet gone to zero.
+        if self.io_loop in self.__class__.instance_map:
+            loop_instance_map = self.__class__.instance_map[self.io_loop]
+            if self._instance_key in loop_instance_map:
+                del loop_instance_map[self._instance_key]
+            if not loop_instance_map:
+                del self.__class__.instance_map[self.io_loop]
+
     def __del__(self):
-        self.close()
+        with self._refcount_lock:
+            # Make sure we actually close no matter if something
+            # went wrong with our ref counting
+            self._refcount = 1
+        try:
+            self.close()
+        except socket.error as exc:
+            if exc.errno != errno.EBADF:
+                # If its not a bad file descriptor error, raise
+                raise
 
     def _package_load(self, load):
         return {
@@ -475,7 +518,7 @@ class AsyncTCPPubChannel(salt.transport.mixins.auth.AESPubClientMixin, salt.tran
                     'tok': self.tok,
                     'data': data,
                     'tag': tag}
-            req_channel = salt.utils.async.SyncWrapper(
+            req_channel = salt.utils.asynchronous.SyncWrapper(
                 AsyncTCPReqChannel, (self.opts,)
             )
             try:
@@ -484,6 +527,9 @@ class AsyncTCPPubChannel(salt.transport.mixins.auth.AESPubClientMixin, salt.tran
                 log.info('fire_master failed: master could not be contacted. Request timed out.')
             except Exception:
                 log.info('fire_master failed: %s', traceback.format_exc())
+            finally:
+                # SyncWrapper will call either close() or destroy(), whichever is available
+                del req_channel
         else:
             self._reconnected = True
 
@@ -566,6 +612,11 @@ class TCPReqServerChannel(salt.transport.mixins.auth.AESReqServerMixin, salt.tra
                     raise exc
             self._socket.close()
             self._socket = None
+        if hasattr(self.req_server, 'stop'):
+            try:
+                self.req_server.stop()
+            except Exception as exc:
+                log.exception('TCPReqServerChannel close generated an exception: %s', str(exc))
 
     def __del__(self):
         self.close()
@@ -597,23 +648,22 @@ class TCPReqServerChannel(salt.transport.mixins.auth.AESReqServerMixin, salt.tra
         self.payload_handler = payload_handler
         self.io_loop = io_loop
         self.serial = salt.payload.Serial(self.opts)
-        if USE_LOAD_BALANCER:
-            self.req_server = LoadBalancerWorker(self.socket_queue,
-                                                 self.handle_message,
-                                                 io_loop=self.io_loop,
-                                                 ssl_options=self.opts.get('ssl'))
-        else:
-            if salt.utils.platform.is_windows():
-                self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-                _set_tcp_keepalive(self._socket, self.opts)
-                self._socket.setblocking(0)
-                self._socket.bind((self.opts['interface'], int(self.opts['ret_port'])))
-            self.req_server = SaltMessageServer(self.handle_message,
-                                                io_loop=self.io_loop,
-                                                ssl_options=self.opts.get('ssl'))
-            self.req_server.add_socket(self._socket)
-            self._socket.listen(self.backlog)
+        with salt.utils.asynchronous.current_ioloop(self.io_loop):
+            if USE_LOAD_BALANCER:
+                self.req_server = LoadBalancerWorker(self.socket_queue,
+                                                     self.handle_message,
+                                                     ssl_options=self.opts.get('ssl'))
+            else:
+                if salt.utils.platform.is_windows():
+                    self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                    self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+                    _set_tcp_keepalive(self._socket, self.opts)
+                    self._socket.setblocking(0)
+                    self._socket.bind((self.opts['interface'], int(self.opts['ret_port'])))
+                self.req_server = SaltMessageServer(self.handle_message,
+                                                    ssl_options=self.opts.get('ssl'))
+                self.req_server.add_socket(self._socket)
+                self._socket.listen(self.backlog)
         salt.transport.mixins.auth.AESReqServerMixin.post_fork(self, payload_handler, io_loop)
 
     @tornado.gen.coroutine
@@ -636,7 +686,7 @@ class TCPReqServerChannel(salt.transport.mixins.auth.AESReqServerMixin, salt.tra
 
             try:
                 id_ = payload['load'].get('id', '')
-                if '\0' in id_:
+                if str('\0') in id_:
                     log.error('Payload contains an id with a null byte: %s', payload)
                     stream.send(self.serial.dumps('bad load: id contains a null byte'))
                     raise tornado.gen.Return()
@@ -698,6 +748,7 @@ class SaltMessageServer(tornado.tcpserver.TCPServer, object):
     '''
     def __init__(self, message_handler, *args, **kwargs):
         super(SaltMessageServer, self).__init__(*args, **kwargs)
+        self.io_loop = tornado.ioloop.IOLoop.current()
 
         self.clients = []
         self.message_handler = message_handler
@@ -752,15 +803,23 @@ if USE_LOAD_BALANCER:
             super(LoadBalancerWorker, self).__init__(
                 message_handler, *args, **kwargs)
             self.socket_queue = socket_queue
+            self._stop = threading.Event()
+            self.thread = threading.Thread(target=self.socket_queue_thread)
+            self.thread.start()
 
-            t = threading.Thread(target=self.socket_queue_thread)
-            t.start()
+        def stop(self):
+            self._stop.set()
+            self.thread.join()
 
         def socket_queue_thread(self):
             try:
                 while True:
-                    client_socket, address = self.socket_queue.get(True, None)
-
+                    try:
+                        client_socket, address = self.socket_queue.get(True, 1)
+                    except queue.Empty:
+                        if self._stop.is_set():
+                            break
+                        continue
                     # 'self.io_loop' initialized in super class
                     # 'tornado.tcpserver.TCPServer'.
                     # 'self._handle_connection' defined in same super class.
@@ -774,10 +833,9 @@ class TCPClientKeepAlive(tornado.tcpclient.TCPClient):
     '''
     Override _create_stream() in TCPClient to enable keep alive support.
     '''
-    def __init__(self, opts, resolver=None, io_loop=None):
+    def __init__(self, opts, resolver=None):
         self.opts = opts
-        super(TCPClientKeepAlive, self).__init__(
-            resolver=resolver, io_loop=io_loop)
+        super(TCPClientKeepAlive, self).__init__(resolver=resolver)
 
     def _create_stream(self, max_buffer_size, af, addr, **kwargs):  # pylint: disable=unused-argument
         '''
@@ -793,9 +851,10 @@ class TCPClientKeepAlive(tornado.tcpclient.TCPClient):
         _set_tcp_keepalive(sock, self.opts)
         stream = tornado.iostream.IOStream(
             sock,
-            io_loop=self.io_loop,
             max_buffer_size=max_buffer_size)
-        return stream.connect(addr)
+        if tornado.version_info < (5,):
+            return stream.connect(addr)
+        return stream, stream.connect(addr)
 
 
 class SaltMessageClientPool(salt.transport.MessageClientPool):
@@ -855,8 +914,8 @@ class SaltMessageClient(object):
 
         self.io_loop = io_loop or tornado.ioloop.IOLoop.current()
 
-        self._tcp_client = TCPClientKeepAlive(
-            opts, io_loop=self.io_loop, resolver=resolver)
+        with salt.utils.asynchronous.current_ioloop(self.io_loop):
+            self._tcp_client = TCPClientKeepAlive(opts, resolver=resolver)
 
         self._mid = 1
         self._max_messages = int((1 << 31) - 2)  # number of IDs before we wrap
@@ -879,33 +938,32 @@ class SaltMessageClient(object):
             return
         self._closing = True
         if hasattr(self, '_stream') and not self._stream.closed():
-            self._stream.close()
-            if self._read_until_future is not None:
-                # This will prevent this message from showing up:
-                # '[ERROR   ] Future exception was never retrieved:
-                # StreamClosedError'
-                # This happens because the logic is always waiting to read
-                # the next message and the associated read future is marked
-                # 'StreamClosedError' when the stream is closed.
-                self._read_until_future.exc_info()
-                if (not self._stream_return_future.done() and
-                        self.io_loop != tornado.ioloop.IOLoop.current(
-                            instance=False)):
-                    # If _stream_return() hasn't completed, it means the IO
-                    # Loop is stopped (such as when using
-                    # 'salt.utils.async.SyncWrapper'). Ensure that
-                    # _stream_return() completes by restarting the IO Loop.
-                    # This will prevent potential errors on shutdown.
-                    orig_loop = tornado.ioloop.IOLoop.current()
-                    self.io_loop.make_current()
-                    try:
+            # If _stream_return() hasn't completed, it means the IO
+            # Loop is stopped (such as when using
+            # 'salt.utils.asynchronous.SyncWrapper'). Ensure that
+            # _stream_return() completes by restarting the IO Loop.
+            # This will prevent potential errors on shutdown.
+            try:
+                orig_loop = tornado.ioloop.IOLoop.current()
+                self.io_loop.make_current()
+                self._stream.close()
+                if self._read_until_future is not None:
+                    # This will prevent this message from showing up:
+                    # '[ERROR   ] Future exception was never retrieved:
+                    # StreamClosedError'
+                    # This happens because the logic is always waiting to read
+                    # the next message and the associated read future is marked
+                    # 'StreamClosedError' when the stream is closed.
+                    if self._read_until_future.done():
+                        self._read_until_future.exception()
+                    elif self.io_loop != tornado.ioloop.IOLoop.current(instance=False):
                         self.io_loop.add_future(
                             self._stream_return_future,
                             lambda future: self.io_loop.stop()
                         )
                         self.io_loop.start()
-                    finally:
-                        orig_loop.make_current()
+            finally:
+                orig_loop.make_current()
         self._tcp_client.close()
         # Clear callback references to allow the object that they belong to
         # to be deleted.
@@ -945,21 +1003,21 @@ class SaltMessageClient(object):
             if self._closing:
                 break
             try:
-                if (self.source_ip or self.source_port) and tornado.version_info >= (4, 5):
-                    ### source_ip and source_port are supported only in Tornado >= 4.5
-                    # See http://www.tornadoweb.org/en/stable/releases/v4.5.0.html
-                    # Otherwise will just ignore these args
+                kwargs = {}
+                if self.source_ip or self.source_port:
+                    if tornado.version_info >= (4, 5):
+                        ### source_ip and source_port are supported only in Tornado >= 4.5
+                        # See http://www.tornadoweb.org/en/stable/releases/v4.5.0.html
+                        # Otherwise will just ignore these args
+                        kwargs = {'source_ip': self.source_ip,
+                                  'source_port': self.source_port}
+                    else:
+                        log.warning('If you need a certain source IP/port, consider upgrading Tornado >= 4.5')
+                with salt.utils.asynchronous.current_ioloop(self.io_loop):
                     self._stream = yield self._tcp_client.connect(self.host,
                                                                   self.port,
                                                                   ssl_options=self.opts.get('ssl'),
-                                                                  source_ip=self.source_ip,
-                                                                  source_port=self.source_port)
-                else:
-                    if self.source_ip or self.source_port:
-                        log.warning('If you need a certain source IP/port, consider upgrading Tornado >= 4.5')
-                    self._stream = yield self._tcp_client.connect(self.host,
-                                                                  self.port,
-                                                                  ssl_options=self.opts.get('ssl'))
+                                                                  **kwargs)
                 self._connecting_future.set_result(True)
                 break
             except Exception as e:
@@ -1144,14 +1202,14 @@ class Subscriber(object):
         self._closing = True
         if not self.stream.closed():
             self.stream.close()
-            if self._read_until_future is not None:
+            if self._read_until_future is not None and self._read_until_future.done():
                 # This will prevent this message from showing up:
                 # '[ERROR   ] Future exception was never retrieved:
                 # StreamClosedError'
                 # This happens because the logic is always waiting to read
                 # the next message and the associated read future is marked
                 # 'StreamClosedError' when the stream is closed.
-                self._read_until_future.exc_info()
+                self._read_until_future.exception()
 
     def __del__(self):
         self.close()
@@ -1162,7 +1220,8 @@ class PubServer(tornado.tcpserver.TCPServer, object):
     TCP publisher
     '''
     def __init__(self, opts, io_loop=None):
-        super(PubServer, self).__init__(io_loop=io_loop, ssl_options=opts.get('ssl'))
+        super(PubServer, self).__init__(ssl_options=opts.get('ssl'))
+        self.io_loop = io_loop
         self.opts = opts
         self._closing = False
         self.clients = set()
@@ -1201,7 +1260,7 @@ class PubServer(tornado.tcpserver.TCPServer, object):
             clients = self.present[id_]
             clients.add(client)
         else:
-            self.present[id_] = set([client])
+            self.present[id_] = {client}
             if self.presence_events:
                 data = {'new': [id_],
                         'lost': []}
@@ -1347,14 +1406,18 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
         return {'opts': self.opts,
                 'secrets': salt.master.SMaster.secrets}
 
-    def _publish_daemon(self, log_queue=None):
+    def _publish_daemon(self, **kwargs):
         '''
         Bind to the interface specified in the configuration file
         '''
         salt.utils.process.appendproctitle(self.__class__.__name__)
 
+        log_queue = kwargs.get('log_queue')
         if log_queue is not None:
             salt.log.setup.set_multiprocessing_logging_queue(log_queue)
+        log_queue_level = kwargs.get('log_queue_level')
+        if log_queue_level is not None:
+            salt.log.setup.set_multiprocessing_logging_level(log_queue_level)
         salt.log.setup.setup_multiprocessing_logging(log_queue)
 
         # Check if io_loop was set outside
@@ -1386,11 +1449,8 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
 
         # Securely create socket
         log.info('Starting the Salt Puller on %s', pull_uri)
-        old_umask = os.umask(0o177)
-        try:
+        with salt.utils.files.set_umask(0o177):
             pull_sock.start()
-        finally:
-            os.umask(old_umask)
 
         # run forever
         try:
@@ -1398,18 +1458,12 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
         except (KeyboardInterrupt, SystemExit):
             salt.log.setup.shutdown_multiprocessing_logging()
 
-    def pre_fork(self, process_manager):
+    def pre_fork(self, process_manager, kwargs=None):
         '''
         Do anything necessary pre-fork. Since this is on the master side this will
         primarily be used to create IPC channels and create our daemon process to
         do the actual publishing
         '''
-        kwargs = {}
-        if salt.utils.platform.is_windows():
-            kwargs['log_queue'] = (
-                salt.log.setup.get_multiprocessing_logging_queue()
-            )
-
         process_manager.add_process(self._publish_daemon, kwargs=kwargs)
 
     def publish(self, load):
@@ -1429,9 +1483,9 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
             pull_uri = int(self.opts.get('tcp_master_publish_pull', 4514))
         else:
             pull_uri = os.path.join(self.opts['sock_dir'], 'publish_pull.ipc')
-        # TODO: switch to the actual async interface
+        # TODO: switch to the actual asynchronous interface
         #pub_sock = salt.transport.ipc.IPCMessageClient(self.opts, io_loop=self.io_loop)
-        pub_sock = salt.utils.async.SyncWrapper(
+        pub_sock = salt.utils.asynchronous.SyncWrapper(
             salt.transport.ipc.IPCMessageClient,
             (pull_uri,)
         )
