@@ -20,6 +20,7 @@ import hmac
 import logging
 import salt.config
 import re
+import random
 from salt.ext import six
 
 # Import Salt libs
@@ -51,6 +52,8 @@ AWS_RETRY_CODES = [
 ]
 AWS_METADATA_TIMEOUT = 3.05
 
+AWS_MAX_RETRIES = 7
+
 IROLE_CODE = 'use-instance-role-credentials'
 __AccessKeyId__ = ''
 __SecretAccessKey__ = ''
@@ -60,6 +63,21 @@ __Location__ = ''
 __AssumeCache__ = {}
 
 
+def sleep_exponential_backoff(attempts):
+    """
+    backoff an exponential amount of time to throttle requests
+    during "API Rate Exceeded" failures as suggested by the AWS documentation here:
+    https://docs.aws.amazon.com/AWSEC2/latest/APIReference/query-api-troubleshooting.html
+    and also here:
+    https://docs.aws.amazon.com/general/latest/gr/api-retries.html
+    Failure to implement this approach results in a failure rate of >30% when using salt-cloud with
+    "--parallel" when creating 50 or more instances with a fixed delay of 2 seconds.
+    A failure rate of >10% is observed when using the salt-api with an asynchronous client
+    specified (runner_async).
+    """
+    time.sleep(random.uniform(1, 2**attempts))
+
+
 def creds(provider):
     '''
     Return the credentials for AWS signing.  This could be just the id and key
@@ -70,6 +88,8 @@ def creds(provider):
     # Declare globals
     global __AccessKeyId__, __SecretAccessKey__, __Token__, __Expiration__
 
+    ret_credentials = ()
+
     # if id or key is 'use-instance-role-credentials', pull them from meta-data
     ## if needed
     if provider['id'] == IROLE_CODE or provider['key'] == IROLE_CODE:
@@ -89,9 +109,7 @@ def creds(provider):
                 proxies={'http': ''}, timeout=AWS_METADATA_TIMEOUT,
             )
             result.raise_for_status()
-            role = result.text.encode(
-                result.encoding if result.encoding else 'utf-8'
-            )
+            role = result.text
         except (requests.exceptions.HTTPError, requests.exceptions.ConnectionError):
             return provider['id'], provider['key'], ''
 
@@ -109,9 +127,18 @@ def creds(provider):
         __SecretAccessKey__ = data['SecretAccessKey']
         __Token__ = data['Token']
         __Expiration__ = data['Expiration']
-        return __AccessKeyId__, __SecretAccessKey__, __Token__
+
+        ret_credentials = __AccessKeyId__, __SecretAccessKey__, __Token__
     else:
-        return provider['id'], provider['key'], ''
+        ret_credentials = provider['id'], provider['key'], ''
+
+    if provider.get('role_arn') is not None:
+        provider_shadow = provider.copy()
+        provider_shadow.pop("role_arn", None)
+        log.info("Assuming the role: %s", provider.get('role_arn'))
+        ret_credentials = assumed_creds(provider_shadow, role_arn=provider.get('role_arn'), location='us-east-1')
+
+    return ret_credentials
 
 
 def sig2(method, endpoint, params, provider, aws_api_version):
@@ -442,8 +469,8 @@ def query(params=None, setname=None, requesturl=None, location=None,
         )
         headers = {}
 
-    attempts = 5
-    while attempts > 0:
+    attempts = 0
+    while attempts < AWS_MAX_RETRIES:
         log.debug('AWS Request: %s', requesturl)
         log.trace('AWS Request Parameters: %s', params_with_headers)
         try:
@@ -451,7 +478,7 @@ def query(params=None, setname=None, requesturl=None, location=None,
             log.debug('AWS Response Status Code: %s', result.status_code)
             log.trace(
                 'AWS Response Text: %s',
-                result.text.encode(result.encoding if result.encoding else 'utf-8')
+                result.text
             )
             result.raise_for_status()
             break
@@ -461,15 +488,14 @@ def query(params=None, setname=None, requesturl=None, location=None,
 
             # check to see if we should retry the query
             err_code = data.get('Errors', {}).get('Error', {}).get('Code', '')
-            if attempts > 0 and err_code and err_code in AWS_RETRY_CODES:
-                attempts -= 1
+            if attempts < AWS_MAX_RETRIES and err_code and err_code in AWS_RETRY_CODES:
+                attempts += 1
                 log.error(
                     'AWS Response Status Code and Error: [%s %s] %s; '
                     'Attempts remaining: %s',
                     exc.response.status_code, exc, data, attempts
                 )
-                # Wait a bit before continuing to prevent throttling
-                time.sleep(2)
+                sleep_exponential_backoff(attempts)
                 continue
 
             log.error(
@@ -488,11 +514,7 @@ def query(params=None, setname=None, requesturl=None, location=None,
             return {'error': data}, requesturl
         return {'error': data}
 
-    response = result.text.encode(
-        result.encoding if result.encoding else 'utf-8'
-    )
-
-    root = ET.fromstring(response)
+    root = ET.fromstring(result.text)
     items = root[1]
     if return_root is True:
         items = root
