@@ -5,8 +5,10 @@ Helpful decorators for module writing
 
 # Import python libs
 from __future__ import absolute_import, print_function, unicode_literals
+import errno
 import inspect
 import logging
+import subprocess
 import sys
 import time
 from functools import wraps
@@ -21,6 +23,10 @@ from salt.log import LOG_LEVELS
 # Import 3rd-party libs
 from salt.ext import six
 
+IS_WINDOWS = False
+if getattr(sys, 'getwindowsversion', False):
+    IS_WINDOWS = True
+
 log = logging.getLogger(__name__)
 
 
@@ -28,7 +34,7 @@ class Depends(object):
     '''
     This decorator will check the module when it is loaded and check that the
     dependencies passed in are in the globals of the module. If not, it will
-    cause the function to be unloaded (or replaced)
+    cause the function to be unloaded (or replaced).
     '''
     # kind -> Dependency -> list of things that depend on it
     dependency_dict = defaultdict(lambda: defaultdict(dict))
@@ -40,6 +46,8 @@ class Depends(object):
 
         An example use of this would be:
 
+        .. code-block:: python
+
             @depends('modulename')
             def test():
                 return 'foo'
@@ -49,24 +57,46 @@ class Depends(object):
             @depends('modulename', fallback_function=function)
             def test():
                 return 'foo'
-        '''
 
+        .. code-block:: python
+
+        This can also be done with the retcode of a command, using the
+        ``retcode`` argument:
+
+            @depends('/opt/bin/check_cmd', retcode=0)
+            def test():
+                return 'foo'
+
+        It is also possible to check for any nonzero retcode using the
+        ``nonzero_retcode`` argument:
+
+            @depends('/opt/bin/check_cmd', nonzero_retcode=True)
+            def test():
+                return 'foo'
+
+        .. note::
+            The command must be formatted as a string, not a list of args.
+            Additionally, I/O redirection and other shell-specific syntax are
+            not supported since this uses shell=False when calling
+            subprocess.Popen().
+
+        '''
         log.trace(
-            'Depends decorator instantiated with dep list of %s',
-            dependencies
+            'Depends decorator instantiated with dep list of %s and kwargs %s',
+            dependencies, kwargs
         )
         self.dependencies = dependencies
-        self.fallback_function = kwargs.get('fallback_function')
+        self.params = kwargs
 
     def __call__(self, function):
         '''
         The decorator is "__call__"d with the function, we take that function
         and determine which module and function name it is to store in the
-        class wide depandancy_dict
+        class wide dependency_dict
         '''
         try:
-            # This inspect call may fail under certain conditions in the loader. Possibly related to
-            # a Python bug here:
+            # This inspect call may fail under certain conditions in the loader.
+            # Possibly related to a Python bug here:
             # http://bugs.python.org/issue17735
             frame = inspect.stack()[1][0]
             # due to missing *.py files under esky we cannot use inspect.getmodule
@@ -74,15 +104,33 @@ class Depends(object):
             _, kind, mod_name = frame.f_globals['__name__'].rsplit('.', 2)
             fun_name = function.__name__
             for dep in self.dependencies:
-                self.dependency_dict[kind][dep][(mod_name, fun_name)] = \
-                        (frame, self.fallback_function)
+                self.dependency_dict[kind][dep][(mod_name, fun_name)] = (frame, self.params)
         except Exception as exc:
-            log.error(
+            log.exception(
                 'Exception encountered when attempting to inspect frame in '
-                'dependency decorator: %s', exc
+                'dependency decorator'
             )
         return function
 
+    @staticmethod
+    def run_command(dependency, mod_name, func_name):
+        full_name = '{0}.{1}'.format(mod_name, func_name)
+        log.trace('Running \'%s\' for \'%s\'', dependency, full_name)
+        if IS_WINDOWS:
+            args = salt.utils.args.shlex_split(dependency, posix=False)
+        else:
+            args = salt.utils.args.shlex_split(dependency)
+        log.trace('Command after shlex_split: %s', args)
+        proc = subprocess.Popen(args,
+                                shell=False,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.STDOUT)
+        output = proc.communicate()[0]
+        retcode = proc.returncode
+        log.trace('Output from \'%s\': %s', dependency, output)
+        log.trace('Retcode from \'%s\': %d', dependency, retcode)
+        return retcode
+
     @classmethod
     def enforce_dependencies(cls, functions, kind):
         '''
@@ -92,24 +140,53 @@ class Depends(object):
         are missing dependencies.
         '''
         for dependency, dependent_dict in six.iteritems(cls.dependency_dict[kind]):
-            for (mod_name, func_name), (frame, fallback_function) in six.iteritems(dependent_dict):
+            for (mod_name, func_name), (frame, params) in six.iteritems(dependent_dict):
+                if 'retcode' in params or 'nonzero_retcode' in params:
+                    try:
+                        retcode = cls.run_command(dependency, mod_name, func_name)
+                    except OSError as exc:
+                        if exc.errno == errno.ENOENT:
+                            log.trace(
+                                'Failed to run command %s, %s not found',
+                                dependency, exc.filename
+                            )
+                        else:
+                            log.trace(
+                                'Failed to run command \'%s\': %s', dependency, exc
+                            )
+                        retcode = -1
+
+                    if 'retcode' in params:
+                        if params['retcode'] == retcode:
+                            continue
+
+                    elif 'nonzero_retcode' in params:
+                        if params['nonzero_retcode']:
+                            if retcode != 0:
+                                continue
+                        else:
+                            if retcode == 0:
+                                continue
+
                 # check if dependency is loaded
-                if dependency is True:
+                elif dependency is True:
                     log.trace(
                         'Dependency for %s.%s exists, not unloading',
                         mod_name, func_name
                     )
                     continue
+
                 # check if you have the dependency
-                if dependency in frame.f_globals \
+                elif dependency in frame.f_globals \
                         or dependency in frame.f_locals:
                     log.trace(
                         'Dependency (%s) already loaded inside %s, skipping',
                         dependency, mod_name
                     )
                     continue
+
                 log.trace(
-                    'Unloading %s.%s because dependency (%s) is not imported',
+                    'Unloading %s.%s because dependency (%s) is not met',
                     mod_name, func_name, dependency
                 )
                 # if not, unload the function
@@ -126,6 +203,7 @@ class Depends(object):
                         continue
 
                     try:
+                        fallback_function = params.get('fallback_function')
                         if fallback_function is not None:
                             functions[mod_key] = fallback_function
                         else:
@@ -216,22 +294,13 @@ class _DeprecationDecorator(object):
 
     def _get_args(self, kwargs):
         '''
-        Extract function-specific keywords from all of the kwargs.
+        Discard all keywords which aren't function-specific from the kwargs.
 
         :param kwargs:
         :return:
         '''
         _args = list()
-        _kwargs = dict()
-
-        if '__pub_arg' in kwargs:  # For modules
-            for arg_item in kwargs.get('__pub_arg', list()):
-                if type(arg_item) == dict:
-                    _kwargs.update(arg_item.copy())
-                else:
-                    _args.append(arg_item)
-        else:
-            _kwargs = kwargs.copy()  # For states
+        _kwargs = salt.utils.args.clean_kwargs(**kwargs)
 
         return _args, _kwargs
 
